#!/usr/bin/env python
# -*- coding: utf-8 -*-
if __name__ == '__main__':
    #try:
    #    import psyco
    #    psyco.full()
    #except ImportError:
    #    pass
    from dynts import runtests
    runtests()


from dynts.dsl.ast import *        

def p_expression_binop(p):
    '''expression : expression PLUS expression
                  | expression MINUS expression
                  | expression TIMES expression
                  | expression DIVIDE expression
                  | expression EQUAL expression
                  | expression CONCAT expression
                  | expression SPLIT expression'''
    v = p[2]
    if v == '+':
        p[0] = PlusOp(p[1],p[3])
    elif v == '-':
        p[0] = MinusOp(p[1],p[3])
    elif v == '*':
        p[0] = MultiplyOp(p[1],p[3])
    elif v == '/':
        p[0] = DivideOp(p[1],p[3])
    elif v == '=':
        p[0] = EqualOp(p[1],p[3])
    elif v == settings.concat_operator:
        p[0] = ConcatenationOp(p[1],p[3])
    elif v == settings.separator_operator:
        p[0] = SplittingOp(p[1],p[3])
    elif v == settings.field_operator:
        p[0] = Symbol(p[1], field = p[3])
        

def p_expression_group(p):
    '''expression : LPAREN expression RPAREN
                  | LSQUARE expression RSQUARE'''
    v = p[1]
    if v == '(':
        p[0] = functionarguments(p[2])
    elif v == '[':
        p[0] = tsentry(p[2])

def p_expression_number(p):
    '''expression : NUMBER'''
    p[0] = Number(p[1])
    
def p_expression_id(p):
    '''expression : ID'''
    p[0] = Symbol(p[1])
    
def p_expression_id_number1(p):
    '''expression : NUMBER ID'''
    p[0] = Symbol('%s%s' % (p[1],p[2]))
    
def p_expression_string(p):
    '''expression : QUOTE expression QUOTE'''
    p[0] = String(p[2])
    
def p_expression_function(p):
    '''expression : FUNCTION LPAREN expression RPAREN'''
    func = p[1]
    p[0] = Function(p[1],p[3],p[2],p[4])
    
def p_expression_bad_function(p):
    '''expression : ID LPAREN expression RPAREN'''
    raise FunctionDoesNotExist(str(p[1]))

def p_error(p):
    raise ExpressionError('Could not parse %s' % p)
    



from datetime import date, timedelta
from random import uniform, randint
import numpy as ny

def_converter = lambda x: x
def_generator = lambda x : uniform(0,1)

__all__ = ['datepopulate','populate','randomts',
           'randomwalk']

class gdata(object):
    
    def __init__(self, data):
        self.data = data
        
    def __iter__(self):
        return (v for v in self.data)
        

def datepopulate(size = 10, start = None, delta = 1):
    dt = start or date.today() - timedelta(days = delta*(size-1))
    td = timedelta(days=delta)
    data = []
    s  = 0
    while s < size:
        data.append(dt)
        dt += td
        s  += 1
    return gdata(data)
        
def populate(size = 10, cols = 1, generator = None):
    generator = generator or def_generator
    data = ny.ndarray([size,cols])
    for c in xrange(0,cols):
        data[:,c] = [generator(i) for i in xrange(0,size)]
    return data


def randomts(size = 100, cols = 1, start = None, delta = 1,
             generator = None, backend=None, name='randomts'):
    from dynts import timeseries
    dates = datepopulate(size,start=start,delta=delta)
    data  = populate(size,cols=cols,generator=generator)
    return timeseries(name=name,backend=backend,date=dates,data=data)

def randomwalk(size = 100, cols = 1, start = None, delta = 1,
               backend=None, name='randomwalk', sigma = 0.1, mu = 0.0):
    '''Create a random walk timeseries'''
    pass

    


import matplotlib.pyplot as plt

def toplot(ts, filename = None, **kwargs):
    '''To plot formatter'''
    fig = plt.figure()
    ax = fig.add_subplot(111)
    dates = list(ts.dates())
    ax.plot(dates, ts.values())
    #ax.xaxis.set_major_locator(years)
    #ax.xaxis.set_major_formatter(yearsFmt)
    #ax.xaxis.set_minor_locator(months)
    #datemin = datetime.date(r.date.min().year, 1, 1)
    #datemax = datetime.date(r.date.max().year+1, 1, 1)
    #ax.set_xlim(datemin, datemax)
    
    # format the coords message box
    #def price(x): return '$%1.2f'%x
    #ax.format_xdata = mdates.DateFormatter('%Y-%m-%d')
    #ax.format_ydata = price
    ax.grid(True)
    
    # rotates and right aligns the x labels, and moves the bottom of the
    # axes up to make room for them
    fig.autofmt_xdate()
    return plt
    



class DyntsException(Exception):
    '''Base class of exceptions raised by ``dynts``'''
    pass

class InvalidBackEnd(DyntsException):
    '''A :class:`DyntsException` exception raised when
an invalid :class:`dynts.TimeSeries` bcakend name is given.'''
    pass

class MissingPackage(DyntsException):
    pass

class MissingDataProvider(DyntsException):
    '''A :class:`DyntsException` exception raised when
a Data provider is not available'''
    pass


class BadSymbol(DyntsException):
    '''A :class:`DyntsException` exception raised when
an exception occurs during parsing of a :class:`dynts.dsl.Symbol`'''
    pass


class FormattingException(DyntsException):
    '''A :class:`DyntsException` exception raised when
an exception occurs during formatting of a :class:`dynts.TimeSeries`'''
    pass


class DateNotFound(DyntsException):
    '''A :class:`DyntsException` exception raised when 
    a date is not found in a :class:`dynts.TimeSeries`.'''
    pass


class DyntsOutOfBound(DyntsException):
    '''A :class:`DyntsException` exception raised when 
    trying to access :class:`dynts.TimeSeries` outside its dates range.'''
    pass


class RightOutOfBound(DyntsOutOfBound):
    '''A :class:`DyntsOutOfBound` exception raised when 
    trying to access :class:`dynts.TimeSeries` after the end date.'''
    pass


class LeftOutOfBound(DyntsOutOfBound):
    '''A :class:`DyntsOutOfBound` exception raised when 
    trying to access :class:`dynts.TimeSeries` before the start date.'''
    pass


class ExpressionError(DyntsException):
    '''A :class:`DyntsException` exception raised when errors occur during
dsl language translation.
    '''

    
class CouldNotParse(ExpressionError):
    
    def __init__(self, f):
        msg = 'Failed to parse expression "%s"' % (f)
        super(CouldNotParse,self).__init__(msg)

    
class FunctionError(ExpressionError):
    '''Function Error'''

        
class FunctionInternalError(FunctionError):
    '''Function Error'''
    def __init__(self, function, msg):
        msg = 'Error in function "%s":' % (function,msg)
        super(FunctionInternalError,self).__init__(msg)
    
class FunctionDoesNotExist(FunctionError):
    '''
    Function does not exist
    '''
    def __init__(self, function):
        msg = 'Function "%s" does not exist' % function
        super(FunctionDoesNotExist,self).__init__(msg)
        
class FunctionTypeError(FunctionError):
    
    def __init__(self, function, msg):
        msg = 'Argument error in function "%s": %s' % (function,msg)
        super(FunctionTypeError,self).__init__(msg)
        
        
class BadConcatenation(ExpressionError):
    '''Error cause by bad concatentaiuon types'''
    def __init__(self, type1, type2):
        msg = 'Failed to concatenate %s with %s' % (type1,type2)
        super(BadConcatenation,self).__init__(msg)



import inspect
from UserDict import UserDict


class FunctionRegistry(UserDict):
    
    def __init__(self):
        self.data = {}
        
    def register(self, function):
        """Register a function in the function registry.
        The function will be automatically instantiated if not already an
        instance.
        """
        function = inspect.isclass(function) and function() or function
        name = function.name
        self.data[name] = function

    def unregister(self, name):
        """Unregister function by name.
        """
        try:
            name = name.name
        except AttributeError:
            pass
        return self.pop(name,None)


class ASTFunctionMeta(type):
    '''Meta class magic'''    
    def __new__(cls, name, bases, attrs):
        global function_registry
        super_new = super(ASTFunctionMeta, cls).__new__

        # Abstract class, remove the abstract attribute so
        # any class inheriting from this won't be abstract by default.
        if attrs.pop("abstract", None) or not attrs.get("autoregister", True):
            return super_new(cls, name, bases, attrs)

        # Automatically generate missing name.
        function_name = attrs.get("name",None) or name
        function_name = function_name.lower()
        attrs["name"] = function_name
        if function_name not in function_registry:
            function_cls = super_new(cls, name, bases, attrs)
            function_registry.register(function_cls)
        return function_registry[function_name].__class__
    
    
class FunctionBase(object):
    '''Base class for a timeseries function implementation.
The only member function to implement is the ``__call__`` method.

    .. function:: __call__(args, **kwargs)
        
        where *args* is a list of arguments (timeseries or other objects) and
        *kwargs* is a dictionary of input parameters.
        For example, the rolling-standard deviation is defined as::
            
            std(expression,window=20)
    '''
    __metaclass__ = ASTFunctionMeta
    abstract = True
    
    def __str__(self):
        return self.name

    def __repr__(self):
        return self.name

function_registry = FunctionRegistry()





# -*- coding: utf-8 -*-
import cStringIO
import csv
from itertools import izip

from dynts.exceptions import FormattingException
from dynts.backends import istimeseries
from dynts.utils import asarray

default_converter = lambda x : x.isoformat()


def tsiterator(ts, dateconverter = None, desc = None):
    dateconverter = dateconverter or default_converter
    yield ['Date']+ts.names()
    for dt,value in ts.items(desc = desc):
        dt = dateconverter(dt)
        yield [dt]+list(value)


class BaseFormatter(object):
    type = None
    default = False


class ToCsv(BaseFormatter):
    type = 'csv'
    
    def __call__(self, ts, filename = None, **kwargs):
        '''Returns CSV representation of a :class:`dynts.TimeSeries`.'''
        stream = cStringIO.StringIO()
        _csv = csv.writer(stream)
    
        for row in tsiterator(ts):
            _csv.writerow(row)
    
        return stream.getvalue()


class ToFlot(BaseFormatter):
    type = 'json'
    default = True
    def __call__(self, ts, container = None, desc = False, **kwargs):
        '''Dump timeseries as a JSON string compatible with ``flot``'''
        from dynts.web import flot
        from dynts.conf import settings
        result = container or flot.MultiPlot()
        if istimeseries(ts):
            res = flot.Flot(ts.name, type = 'timeseries')
            dates  = asarray(ts.dates())
            missing = settings.ismissing
            for name,serie in izip(ts.names(),ts.series()):
                data = []
                for dt,val in izip(dates,serie):
                    if not missing(val):
                        data.append([flot.pydate2flot(dt),val])
                serie = flot.Serie(label = name, data = data)
                res.add(serie)
        else:
            res = flot.Flot(ts.name)
            for name,serie in izip(ts.names(),ts.series()):
                serie = flot.Serie(label = serie.name,
                                   data = serie.data,
                                   lines = {'show':serie.lines},
                                   points = {'show':serie.points})
                res.add(serie)
        result.add(res)
        return result


class ToJsonVba(BaseFormatter):
    '''A JSON Formatter which can be used to serialize data to
VBA. For unserializing check http://code.google.com/p/vba-json/

The unserializer is also included in the directory extras'''
    type = 'json'
    def __call__(self, ts, container = None, desc = False, **kwargs):
        '''Dump timeseries as a JSON string VBA-Excel friendly'''
        from ccy import date2juldate
        from dynts.utils.anyjson import JSONdatainfo
        if istimeseries(ts):
            return JSONdatainfo(list(tsiterator(ts, dateconverter=date2juldate, desc = desc)),
                                info = ts.info)
        else:
            raise NotImplementedError


class ToXls(BaseFormatter):
    type = 'xls'
    
    def __call__(self, ts, filename = None, title = None, raw = False, **kwargs):
        '''Dump the timeseries to an xls representation.
This function requires the python xlwt__ package.
    
__ http://pypi.python.org/pypi/xlwt'''
        try:
            import xlwt
        except ImportError:
            raise FormattingException('To save the timeseries as a spreadsheet, the xlwt python library is required.')
        
        
        if isinstance(filename,xlwt.Workbook):
            wb = filename
        else:
            wb = xlwt.Workbook()
        title = title or ts.name
        stream = cStringIO.StringIO()
        sheet = wb.add_sheet(title)
        for i,row in enumerate(tsiterator(ts)):
            for j,col in enumerate(row):
                sheet.write(i,j,str(col))
        
        if raw:
            return wb
        else:
            stream = cStringIO.StringIO()
            wb.save(stream)
            return stream.getvalue()
    

class ToPlot(BaseFormatter):
    type = 'python'
       
    def __call__(self, ts, **kwargs):
        try:
            import tsplot
            return tsplot.toplot(ts, **kwargs)
        except ImportError:
            raise FormattingException('To plot timeseries, matplotlib is required.')
    
    
            
        


from dynts.tests import tsinterface

class TestZooTS(tsinterface.TestTS):
    backend = 'zoo'

class TestNumpyTS(tsinterface.TestTS):
    backend = 'numpy'


#class TestRmetricsTS(tsbase.TestTS):
#    backend = 'rmetrics'
    


from datetime import datetime, date

from dynts.utils.anyjson import JSONdatainfo, JSONobject

EPOCH = 1970
_EPOCH_ORD = date(EPOCH, 1, 1).toordinal()


class MultiPlot(JSONdatainfo):
    '''Class holding several plots. A plot can be a timeseries or xy-type.'''
    def __init__(self, flot = None, info = None):
        super(MultiPlot,self).__init__(data = [], info = info)
        self.add(flot)
        
    def add(self, flot):
        if isinstance(flot,Flot):
            self.data.append(flot)
        
    def todict(self):
        return {'type': 'multiplot',
                'info': self.info,
                'plots': [plot.todict() for plot in self.data]}
    

class Flot(JSONobject):
    '''A single plot'''
    allowed_types = ['xy','timeseries']
    def __init__(self, name = '', type = None):
        if type not in self.allowed_types:
            type = 'xy'
        self.name   = name
        self.type   = type
        self.series = []
        
    def add(self, serie):
        if isinstance(serie,Serie):
            self.series.append(serie)
        
    def todict(self):
        od = super(Flot,self).todict()
        od['series'] = [serie.todict() for serie in self.series]
        return od


class Serie(JSONobject):
    
    def __init__(self, label = '', data = None, **kwargs):
        self.label = label
        if data is None:
            data = []
        self.data = data
        for k,v in kwargs.items():
            setattr(self,k,v)


def pydate2flot(dte):
    year, month, day, hour, minute, second = dte.timetuple()[:6]
    days = date(year, month, 1).toordinal() - _EPOCH_ORD + day - 1
    hours = days*24 + hour
    minutes = hours*60 + minute
    seconds = minutes*60 + second
    if isinstance(dte,datetime):
        return 1000*seconds + 0.001*dte.microsecond
    else:
        return 1000*seconds




#
# TimeSeries Backend based on numpy
#
#
from collections import deque
from itertools import izip
import numpy as ny

import dynts
from dynts.utils import rollingOperation, laggeddates, asarray


arraytype = ny.ndarray


arrayfunc = lambda func, *args : [func(*items) for items in izip()]


_functions = {'min':min,
              'max':max}


def rollsingle(self, func, window = 20, name = None, **kwargs):
    '''Efficient rolling window calculation for min, max type functions'''
    rolling = lambda serie : list(getattr(rollingOperation(serie,window),func)())
    data = ny.array([rolling(serie) for serie in self.series()])
    name = name or '%s(%s,window=%s)' % (func,self.name,window)
    return self.clone(self.dates()[window-1:], data.transpose(), name = name)


def days(d1,d0):
    t = d1 - d0
    return t.days + (t.seconds + 0.000001*t.microseconds)/86400.0


class TimeSeries(dynts.TimeSeries):
    '''A timeserie based on numpy'''
    type = 'numpy'
    
    def make(self, date, data, raw = False):
        if date is None:
            self._date = None
            self._data = None
        else:
            self._date = asarray(date)
            data = asarray(data)
            if len(data.shape) == 1:
                data = data.reshape(len(data),1)
            self._data = data
    
    @property
    def shape(self):
        return self._data.shape
    
    def __getitem__(self, i):
        return self._data[i]
    
    def values(self, desc = None):
        if desc == True:
            return reversed(self._data)
        else:
            return self._data
    
    def dates(self, desc = None):
        if desc == True:
            return reversed(self._date)
        else:
            return self._date
    
    def keys(self, desc = None):
        return self.dates(desc = desc)
    
    def start(self):
        return self._date[0]
    
    def end(self):
        return self._date[-1]
    
    def isregular(self):
        dates = self.dates().__iter__()
        d0 = dates.next()
        d1 = dates.next()
        dt = d1 - d0
        for d2 in dates:
            if d2 - d1 == dt:
                d1 = d2
                continue
            return False
        return True
    
    def frequency(self):
        freq = 0;
        for d1,d0 in laggeddates(self):
            freq += days(d1,d0) 
        return freq/(len(self)-1)
    
    def window(self, start, end):
        b = self.asbtree()
        i1 = b.find_ge(start)
        i2 = b.find_le(end)
        return self.clone(self._date[i1:i2+1],
                          self._data[i1:i2+1])
    
    def merge(self, tserie, fill = float("nan"), **kwargs):
        if dynts.istimeseries(tserie):
            tserie = [tserie]
        alldates = set(self.dates())
        hash     = self.ashash()
        thashes  = [(hash,ny.array([fill]*self.count()))]
        for ts in tserie:
            alldates = alldates.union(ts.dates())
            hash.names.extend(ts.names())
            thashes.append((ts.ashash(),ny.array([fill]*ts.count())))
        stack = ny.hstack
        mdt = lambda dt: stack((h.get(dt,ln) for h,ln in thashes))
        for dt in alldates:
            hash[dt] = mdt(dt)
        return hash.getts()
    
    def log(self):
        pass
    
    def delta(self, k = 1, name = None):
        self.precondition(k<len(self) and k > 0,dynts.DyntsOutOfBound)
        v = self._data[k:] - self._data[:-k]
        name = name or 'delta(%s,%s)' % (self.name,k)
        return self.clone(self._date[k:],v,name)
    
    def _rollapply(self, func, window = 20, **kwargs):
        fs = _functions.get(func,None)
        if fs:
            return rollsingle(self, func, window = window, **kwargs)
            
    


import unittest

import dynts
from dynts.utils.anyjson import json

class TestFlot(unittest.TestCase):
    
    def testFlot1(self):
        ts = dynts.evaluate('YHOO,GOOG').dump('flot')
        dts = ts.todict()
        self.assertEqual(dts['type'],'multiplot')
        self.assertEqual(len(dts['plots']),1)
        plot = dts['plots'][0]
        self.assertEqual(plot['type'],'timeseries')
        self.assertEqual(len(plot['series']),2)
        data = json.dumps(dts)
        


from math import isnan
toupper = lambda x : str(x).upper()

class Settings(object):
    '''The setting class contains configuration parameters used in dynts.
    
    .. attribute:: backend
    
        the default :class:`dynts.TimeSeries` backend class, Default ``zoo``.
        
    .. attribute:: concat_operator
    
        The operator for concatenating expressions. Default ``,``.
    
    .. attribute:: default_loader
    
        Default :class:`dynts.data.TimeSerieLoader` class. Default ``None``.
        If this is ``None`` the :class:`dynts.data.TimeSerieLoader` class will be used as loder.
        
    .. attribute:: default_provider
    
        Default :class:`dynts.data.DataProvider` code. Default ``"YAHOO"``.
        
    .. attribute:: months_history
    
        the default number of months of history. Default: ``12``.
        
    .. attribute:: proxies
    
        dictionary of proxy servers. Default ``{}``.
        If you need to use a proxy server to access the web::
        
            from dynts.conf import settings
            settings.proxies['http'] = 'http://yourproxy.com:80'
                
To change settings::
    
    from dynts.conf import settings
    
    settings.default_provider = 'GOOGLE'
    '''
    def __init__(self):
        self.backend = 'zoo'
        self.splittingnames     = '__'
        self.concat_operator    = ','
        self.separator_operator = '|'
        self.default_provider   = 'YAHOO'
        self.default_loader     = None
        self.months_history     = 12
        self.proxies = {}
        self.symboltransform = toupper
        self.default_daycounter = 'ACT/365'
        self.missing_value      = float('nan')
        
    @property
    def special_operators(self):
        return [self.concat_operator,self.separator_operator]
    
    def getdc(self):
        import ccy
        return ccy.getdc(self.default_daycounter)
    
    def ismissing(self, value):
        return isnan(value)
    
        
        
settings = Settings()





class DataProvider(object):
    '''Interface class for Data Providers.
    
    .. attribute:: code
    
        The string code for the provider.
        This attribute is obtained from the class name in upper case.'''
        
    def __repr__(self):
        return self.code + ' financial data provider'
    
    def load(self, ticker, startdate, enddate, field, logger, backend):
        '''This is the function to implement. It loads the actual data from the data rovider.
This function is not called directly, instead it is called by the 
:meth:`dynts.data.TimeSerieLoader.load`.

* *ticker* string id for the symbol to load (the ticker).
* *startdate* and *enddate* interval to load.
* *field* ``None``, a string or alist of string. Fiels to load.
    If ``None`` the default field for the provider should load.
    
It should return either an instance of :class:`dynts.TimeSeries` or
a dictionary of the form::

    {'date': [list of dates],
     'field1': [list of values for field1],
     ...
     'fieldN': [list of values for field1N]}

* *logger* instance of :class:logging.Logger.
* **backend* :class:`dynts.TimeSeries` backend name.
        '''
        raise NotImplementedError
    
    def isconnected(self):
        '''Return ``True`` if data connection is available
        '''
        return True
    
    def __get_code(self):
        return self.__class__.__name__.upper()
    code = property(fget = __get_code)
    
    def allfields(self, ticker = None):
        '''Return a list of all fields available for the providers.
The first of the list will be treated as the default field.'''
        raise NotImplementedError
    
    def weblink(self, ticker):
        '''Optional web link for a given *ticker*. Default return ``None``.
        '''
        return None
    
    def external(self):
        return True
    
    def hasfeed(self, live = False):
        return not live
    
    def connect(self):
        pass
    




from itertools import izip

crossoperator = lambda func,*args : [func(*vals) for vals in izip(*args)]

scalarasiter = lambda x: x if hasattr(x,'__iter__') else [x] 


__all__ = ['cross','asarray','ascolumn','assimple',
           'scalarasiter']


def asarray(x):
    '''Convert ``x`` into a ``numpy.ndarray``.'''
    from numpy import array, ndarray
    iterable = scalarasiter(x)
    if isinstance(iterable,ndarray):
        return iterable
    else:
        if not hasattr(iterable,'__len__'):
            iterable = list(iterable)
        return array(iterable)


def ascolumn(x):
    '''Convert ``x`` into a ``column``-type ``numpy.ndarray``.'''
    x = asarray(x)
    return x if len(x.shape) >= 2 else x.reshape(len(x),1)

        
def assimple(x):
    if hasattr(x,'__iter__'):
        try:
            len(x)
        except:
            x = list(x)
        if len(x) == 1:
            return x[0]
        else:
            return x
    else:
        return x


class cross(object):
    '''Cross section wrapper class'''
    min = lambda *args : crossoperator(min,*args)
    max = lambda *args : crossoperator(max,*args)

    def __init__(self, elem):
        self.elem = asarray(elem)
    
    def __iter__(self):
        return iter(self.elem)
    
    def __eq__(self, other):
        return reduce(lambda x,y : x and y[0] == y[1], izip(self.elem,asarray(other)), True)
    
    def __ge__(self, other):
        return reduce(lambda x,y : x and y[0] >= y[1], izip(self.elem,asarray(other)), True)
    
    def __le__(self, other):
        return reduce(lambda x,y : x and y[0] <= y[1], izip(self.elem,asarray(other)), True)

    def __gt__(self, other):
        return not (self <= other)
    
    def __lt__(self, other):
        return not (self >= other)
    


from dynts.dsl import FunctionBase
from dynts import TimeSeries


class Delta(FunctionBase):
    def __call__(self, data, step = 1):
        pass
    
    
class Log(FunctionBase):
    def __call__(self, data, step = 1):
        for k,v in data:
            yield k,log(v)


class ScalarFunction(FunctionBase):
    abstract = True
    def __call__(self, args, window = 20, **kwargs):
        result = []
        for arg in args:
            name = '%s(%s,window=%s)' % (self.name,arg,window)
            ts = self.apply(arg, window = window, name = name, **kwargs)
            result.append(ts)
        if result:
            return result if len(result)>1 else result[0]


class Ma(ScalarFunction):
    """Moving average function"""
    def apply(self, ts, **kwargs):
        return ts.rollmean(**kwargs)
    
    
class Max(ScalarFunction):
    """Moving max function"""
    def apply(self, ts, **kwargs):
        return ts.rollmax(**kwargs)


class Med(ScalarFunction):
    """Moving median function"""
    def apply(self, ts, **kwargs):
        return ts.rollmedian(**kwargs)    
    
    
class Min(ScalarFunction):
    """Moving min function"""
    def apply(self, ts, **kwargs):
        return ts.rollmin(**kwargs)
    
    
class Avol(ScalarFunction):
    '''Annualised volatility'''
    def apply(self, ts, **kwargs):
        return ts.rollavol(**kwargs)
    
    
class Regression(FunctionBase):
    """Calculate the **linear regression** of one series with respect
to one or more series. For example::

    regr(GOOG,YHOO)
    
will calculate

.. math::

    y_i = \beta x_i + \alpha
    
There are two optional parameters:

* *alpha* default is ``1``. If set to zero alpha won't be included in the regression."""        
    name = 'regr'
    
    def __call__(self, input, **kwargs):
        pass
        
    


from itertools import izip
from rpy2.robjects import IntVector

import dynts
from dynts.conf import settings
from dynts.backends.rbase import rts

 
def v2bool(v):
    try:
        return v[0]
    except:
        return False
    

def tozoo(ts):
    if isinstance(ts,TimeSeries):
        if ts.type == 'zoo':
            return ts
        else:
            raise NotImplementedError('Cannot convert %s to zoo timeserie' % ts)
    else:
        raise ValueError('%s is not a timeserie object' % ts)


class TimeSeries(rts):
    type = 'zoo'
    libraries = ['zoo,PerformanceAnalytics']
    
    def factory(self, date, data, raw = False):
        if not raw:
            tdate = self.dateconvert
            date = IntVector([tdate(dt) for dt in date])
        #data = FloatVector(data)
        return self.r['zoo'](data,date)
    
    def colnames(self):
        #TODO: Not working since we did not set names
        return self.r['colnames'](self._ts)
    
    def _rollapply(self, func, window = None, **kwargs):
        fun    = self.r[func]
        return self.rcts('rollapply', window, fun, **kwargs)
    
    def start(self):
        return self.dateinverse(self.rc('start')[0])
        
    def end(self):
        return self.dateinverse(self.rc('end')[0])
    
    def _mergesingle(self, ts, all = True):
        ts = tozoo(ts)
        name = settings.splittingnames.join(self.names() + ts.names())
        return self.rcts('merge', ts._ts, all = all, name = name)
    
    def merge(self, ts, all = True):
        if dynts.istimeseries(ts):
            return self._mergesingle(ts, all = all)
        else:
            rs = self
            for t in ts:
                rs = rs._mergesingle(t, all = all)
            return rs
    


import time
from datetime import date, datetime
try:
    import json
except:
    import simplejson as json
    
date2timestamp = lambda dte : int(time.mktime(dte.timetuple()))
    
class JSONRPCEncoder(json.JSONEncoder):
    """
    Provide custom serializers for JSON-RPC.
    """
    def default(self, obj):
        if isinstance(obj, date) or isinstance(obj, datetime):
            return date2timestamp(obj)
        else:
            raise exceptions.JSONEncodeException("%r is not JSON serializable" % (obj,))
        

class jsonPickler(object):
    
    def dumps(self, obj, **kwargs):
        return json.dumps(res, cls=JSONRPCEncoder, **kwargs)
    
    def loads(self,sobj):
        return json.loads(sobj)


class JSONobject(object):
    pickler = jsonPickler()
    
    def todict(self):
        d = self.__dict__.copy()
        d.pop('pickler',None)
        return d
    
    def tojson(self):
        return self.pickler.dumps(self.todict())
    
    
class JSONdatainfo(JSONobject):
    
    def __init__(self, data = None, info = None):
        self.data = data
        self.info = info
        



import numpy as np
from dynts.backends import base

_addoper = lambda x,y : x+y
_suboper = lambda x,y : x-y
_muloper = lambda x,y : x*y
_divoper = lambda x,y : x/y

_ops = { 'add' : _addoper,
        'sub' : _suboper,
        'mul' : _muloper,
        'div' : _divoper,
        }


def _get_op(op_name):
    global _ops
    op = _ops.get(op_name, None)
    if op is None:
        msg = 'There is no op called %s. choices are %s' %(op_name, ops)
        raise ValueError(msg)
    return op

def binOp(op, indx, amap, bmap, handle_missing):
    '''
    Combines the values from two map objects using the indx values
    using the op operator. In situations where there is a missing value
    it will use the callable function handle_missing
    '''
    def op_or_missing(id):
        va = amap.get(id, None)
        vb = bmap.get(id, None)
        if va is None or vb is None:
            result = handle_missing() #This should create as many elements as the number of columns!?
        else:
            try:
                result = op(va, vb)
            except Exception, e:
                result = None
            if result == None:
                result = handle_missing()
            return result
    seq_arys = map(op_or_missing, indx)
    data = np.vstack(seq_arys)
    return data

def applyfn(op, v1, v2, handle_missing):
    def op_or_missing(a,b):
        try:
            result = op(a,b)
        except Exception, e:
            result = None
        if result is None:
                result = handle_missing()
        return result
    if len(v1) != len(v2):
        msg = "Vectors %s, %s are different lengths %s, %s" %(v1, v2, len(v1),
                len(v2))
        raise ValueError(msg)
    rt = map(op_or_missing, v1, v2)
    return rt

def is_timeseries(ts_or_scalar):
    result = isinstance(ts_or_scalar, base.TimeSeries)
    return result

def _combine_dts(dts1, dts2, all):
    if all:
        all_dts = set(dts1).union(set(dts2))
    else:
        all_dts = set(dts1).intersection(set(dts2))

    result = list(all_dts)
    result.sort()
    return result

def _toVec(shape, val):
    '''
    takes a single value and creates a vecotor / matrix with that value filled
    in it
    '''
    mat = np.empty(shape)
    mat.fill(val)
    return mat

def _handle_scalar(op, ts, scalar, fill_fn):
    values = ts.values()
    shape = values.shape
    v2 = _toVec(shape, scalar)
    dts = ts.dates()
    result = applyfn(op, values, v2, fill_fn)
    return dts, result

def _handle_ts(op, ts, ts2, all, fill_fn):
    dts1 = ts.dates()
    dts2 = ts2.dates()
    indx = _combine_dts(dts1, dts2, all)
    tsMap = ts.asbtree()
    ts2Map = ts2.asbtree()
    result = binOp(op, indx, tsMap, ts2Map, fill_fn)
    return indx, result

def _handle_ts_or_scalar(op_name, ts, ts_or_scalar, all = True, fill = None):
    op = _get_op(op_name)
    if callable(fill):
        fill_fn = fill
    else:
        fill_fn = lambda : fill

    scalar = not is_timeseries(ts_or_scalar)
    if scalar:
        dts, data = _handle_scalar(op, ts, ts_or_scalar, fill_fn)
    else:
        dts, data = _handle_ts(op, ts, ts_or_scalar, all, fill_fn)
    new_ts = ts.clone(date = dts, data = data)
    return new_ts

def ts_fn(op_name):
    fn = lambda  *args,  **kwargs : _handle_ts_or_scalar(op_name, *args, **kwargs)
    return fn

add = ts_fn('add')
sub = ts_fn('sub')
mul = ts_fn('mul')
div = ts_fn('div')




import logging
from datetime import date, timedelta

from dynts.conf import settings
from dynts.exceptions import *

from gy import DataProvider, google, yahoo


class Silence(logging.Handler):
    def emit(self, record):
        pass

class MissingDataProvider(Exception):
    '''Data provider is not available'''
    pass


def safetodate(dte):
    from ccy import todate
    try:
        return todate(dte)
    except:
        return None


class PreProcessData(object):
    '''data preprocess holder'''
    def __init__(self, intervals = None, result = None):
        self.intervals = intervals
        self.result    = result
        

class TimeSerieLoader(object):
    '''Cordinates the loading of timeseries data into :class:`dynts.dsl.Symbol`.
This class can be overritten by a custom one if required. There are three **hooks**
which can be used to customized its behaviour:

* :func:`dynts.data.TimeSerieLoader.preprocess`
* :func:`dynts.data.TimeSerieLoader.onresult`
* :func:`dynts.data.TimeSerieLoader.onfinishload`

.. attribute:: separator
    
    character for separating ticker, field and vendor. Default ``:``'''

    separator = ':'
    preprocessdata = PreProcessData
    '''Class holding data returned by the :meth:`dynts.data.TimeSerieLoader.preprocess` method.
    It contains two attributes:
    
    * ``intervals`` ``None`` or a tuple of two-elements tuples.
    * ``result`` ``None`` or any intermediate result.
    
    If ``intervals`` is ``None`` or an empty container, the
    :func:`dynts.data.DataProvider.load` method won't be called, and the ``result``
    attribute will be passed to the :meth:`dynts.data.TimeSerieLoader.onresult` method.
    '''
    
    def load(self, providers, symbols, start, end, logger, backend, **kwargs):
        '''Load symbols data.
        
* *providers* Dictionary of registered data providers.
* *symbols* list of symbols to load
* *start* start date
* *end* end date.
* *logger* instance of :class:`logging.Logger`.
* *backend* :class:`dynts.TimeSeries` backend name.

There is no need to override this function, just use one the three hooks
available.
'''
        # Preconditioning on dates
        logger = logger or logging.getLogger(self.__class__.__name__)
        start, end = self.dates(start, end)
        data = {}
        for symbol in symbols:
            # Get ticker, field and provider
            ticker, field, provider = self.parse_symbol(symbol)
            p  = providers.get(provider,None)
            if not p:
                raise MissingDataProvider('data provider %s not available' % provider)
            pre = self.preprocess(ticker, start, end, field, p, logger, backend, **kwargs)
            if pre.intervals:
                result = None
                for st,en in pre.intervals:
                    logger.info('Loading %s from %s. From %s to %s' % (ticker,provider,st,en))
                    res = p.load(ticker, st, en, field, logger, backend, **kwargs)
                    if result is None:
                        result = res
                    else:
                        result.update(res)
            else:
                result = pre.result
            # onresult hook
            result = self.onresult(ticker, field, p, result, logger, backend, **kwargs)
            data[symbol] = result
        # last hook
        return self.onfinishload(data, logger, backend, **kwargs)
    
    def dates(self, start, end):
        '''Pre-conditioning on dates. This function makes sure the *start*
and *end* date are consistent. It never fails and always return a two-element tuple
containing *start*,*end* with *start* less or equal *end*
and *end* never after today.
There should be no reason to override this function.'''
        td    = date.today()
        end   = safetodate(end) or td
        end   = end if end <= td else td
        start = safetodate(start)
        if not start or start > end:
            start = end - timedelta(days=int(round(30.4*settings.months_history)))
        return start,end
    
    def parse_symbol(self, symbol):
        '''Parse a symbol to obtain information regarding ticker, field and provider.
Must return a tuple containing::

    (ticker,fied,provider)
    
For example::

    intc
    intc:open
    intc:volume:google
    intc:google
    
are all valid inputs returning::

    intc,None,yahoo
    intc,open,yahoo
    intc,volume,google
    intc,None,google
    
assuming ``yahoo`` is the provider in :attr:`dynts.conf.Settings.default_provider`.

This function is called before retrieving data. As with previous functions there should be no reason to
override this method. One could use the
:func:`dynts.data.TimeSerieLoader.default_provider_for_ticker` to twick
the behaviour when the provider is not available.
'''
        if not symbol:
            raise BadSymbol("symbol not provided")
        symbol = str(symbol)
        bits = symbol.split(self.separator)
        pnames = providers.keys()
        ticker = symbol
        provider = None
        field = None
        if len(bits) == 2:
            ticker = bits[0]
            if bits[1] in pnames:
                provider = bits[1]
            else:
                field = bits[1]
        elif len(bits) == 3:
            ticker = bits[0]
            if bits[1] in pnames:
                provider = bits[1]
                field = bits[2]
            elif bits[2] in pnames:
                provider = bits[2]
                field = bits[1]
            else:
                raise BadSymbol('Could not parse %s. Unrecognized provider.' % symbol)
        elif len(bits) > 3:
            raise BadSymbol('Could not parse %s.' % symbol)
        
        if provider is None:
            provider = self.default_provider_for_ticker(ticker, field)
        return ticker,field,provider
 
    def default_provider_for_ticker(self, ticker, field):
        '''Calculate the provider when not available in the symbol. By default it returns
:attr:`dynts.conf.Settings.default_provider`.'''
        return settings.default_provider
    
    def getsymbol(self, ticker, field, provider):
        '''Convert *ticker*, *field* and *provider* to symbol code.
The inverse of :meth:`dynts.data.TimeSerieLoader.parse_symbol`.'''
        c = self.separator
        f = '' if not field else '%s%s' % (c,field)
        d = provider == self.default_provider_for_ticker(ticker, field)
        p = '' if d else '%s%s' % (c,provider)
        return '%s%s%s' % (ticker,f,p)
    
    def preprocess(self, ticker, start, end, field, provider, logger, backend, **kwargs):
        '''Preprocess **hook**. This is first loading hook and it is
**called before requesting data** from a dataprovider.
It must return an instance of :attr:`TimeSerieLoader.preprocessdata`.
By default it returns::

    self.preprocessdata(intervals = ((start,end),))
    
It could be overritten to modify the intervals. If the intervals is ``None`` or 
an empty container, the :func:`dynts.data.DataProvider.load` method won't be called,
otherwise it will be called as many times as the number of intervals in the return tuple
(by default once).
'''
        return self.preprocessdata(intervals = ((start, end),))
    
    def onresult(self, ticker, field, provider, result, logger, backend, **kwargs):
        '''Post-processing **hook** for results returned by
calls to :func:`dynts.data.DataProvider.load`, or obtained
from the :meth:`dynts.data.TimeSerieLoader.preprocess` method.
By default return ``result``::
    
    return result
    
It could be used to store data into a cache or database.'''
        return result
    
    def onfinishload(self, data, logger, backend, **kwargs):
        '''Another post-processing **hook** invoked when the loading is finished.
By default retun *data*.'''
        return data


class DataProviders(dict):
    proxies = {}
        
    def load(self, symbols, start = None, end = None, loader = None,
             logger = None,  backend = None, **kwargs):
        loader = loader or settings.default_loader or TimeSerieLoader
        backend = backend or settings.backend
        if isinstance(loader,type):
            loader = loader()
        return loader.load(self,symbols,start,end,logger,backend,**kwargs)
    
    def register(self, provider):
        '''Register a new data provider. *provider* must be an instance of
    DataProvider. If provider name is already available, it will be replaced.'''
        if isinstance(provider,type):
            provider = provider()
        self[provider.code] = provider
        
    def unregister(self, provider):
        '''Unregister an existing data provider. *provider* must be an instance of
    DataProvider. If provider name is already available, it will be replaced.'''
        if isinstance(provider,type):
            provider = provider()
        if isinstance(provider,DataProvider):
            provider = provider.code
        return self.pop(str(provider).upper(),None)
        
    def get_logger(self, logger):
        if logger:
            return logger
        logger = logging.getLogger('dynts.data')
        if not logger.handlers:
            logger.addHandler(Silence())
        return logger

providers = DataProviders()
register = providers.register
unregister = providers.unregister

register(google)
register(yahoo)




import unittest

import dynts
from dynts.data import DataProvider, TimeSerieLoader
from dynts.data import providers, register, unregister


class CustomProvider(DataProvider):
    pass

class CustomLoader(TimeSerieLoader):
    data = {}
    
    def onresult(self, ticker, field, provider, result, logger, backend, **kwargs):
        '''Store result in the class data dictionary'''
        symbol = self.getsymbol(ticker, field, provider.code)
        self.data[symbol] = result
        return result
        


class TestDataProvider(unittest.TestCase):
    
    def testyahoo(self):
        ts = dynts.evaluate('GOOG:yahoo')
        self.assertTrue(ts)
        
    def testProviderRegistration(self):
        register(CustomProvider)
        self.assertEqual(len(providers),3)
        p = providers['CUSTOMPROVIDER']
        self.assertTrue(isinstance(p,CustomProvider))
        unregister('CUSTOMPROVIDER')
        self.assertEqual(len(providers),2)
        p = providers.get('CUSTOMPROVIDER',None)
        self.assertEqual(p,None)
        
    def testCustomLoader(self):
        from dynts.conf import settings
        settings.default_loader = CustomLoader
        dynts.evaluate('BLT:google').unwind()
        data = CustomLoader.data
        self.assertTrue(data)
        self.assertTrue('BLT:GOOGLE' in data)
        
        
        


from dynts.utils import import_module
from dynts.conf import settings
from base import TimeSeries, Formatters, xydata, DynData

BACKENDS = {
    'zoo': 'zoo',
    'rmetrics': 'rmetrics',
    'numpy': 'tsnumpy',
}

istimeseries = lambda value : isinstance(value,TimeSeries)
isxy = lambda value : isinstance(value,xydata)


def timeseries(name = '', backend = None, **kwargs):
    '''Create a new :class:`dynts.TimeSeries` object.'''
    from dynts import InvalidBackEnd
    backend = backend or settings.backend
    bname = BACKENDS.get(backend,None)
    if bname:
        bmodule = 'dynts.backends.%s' % bname
    else:
        bmodule = backend
    module = import_module(bmodule)
    name = name or bmodule
    try:
        factory = getattr(module, 'TimeSeries')
    except AttributeError:
        raise InvalidBackEnd('Could not find a TimeSeries class in module %s' % bmodule)
    return factory(name = name, **kwargs)



from itertools import izip

from dynts.conf import settings
from dynts.utils import laggeddates, ashash, asbtree, asarray
from dynts.backends.xy import *


class TimeSeries(DynData):
    '''A :class:`dynts.DynData` specialisation for timeseries back-ends.
    
    .. attribute:: type
    
        string indicating the backend type (``zoo``, ``rmetrics``, ``numpy``, etc...)
        
    .. attribute:: shape
    
        tuple containing the timeseries dimensions.
    '''
    type = None
    
    def __init__(self, name = '', date = None, data = None, info = None):
        super(TimeSeries,self).__init__(name,info)
        self.make(date,data)
    
    def __len__(self):
        return self.shape[0]
    
    def count(self):
        return self.shape[1]
    
    def asbtree(self):
        '''Return an instance of :class:`dynts.utils.wrappers.asbtree`
which exposes binary tree like functionalities of ``self``.'''
        return asbtree(self)
    
    def ashash(self):
        '''Return an instance of :class:`dynts.utils.wrappers.ashash`
which exposes hash-table like functionalities of ``self``.'''
        return ashash(self)
    
    def dateconvert(self, dte):
        return dte
    
    def dateinverse(self, key):
        return key
    
    def max(self):
        '''Max value'''
        return asarray(self.apply('max')[0])
    
    def min(self):
        '''Max value'''
        return asarray(self.apply('min')[0])
    
    def mean(self):
        '''Mean value'''
        return asarray(self.apply('mean')[0])
    
    def returns(self, k = 1):
        '''Calculate returns as delta(log(self))'''
        return self.log().delta(k)
    
    def dates(self, desc = None):
        '''Returns an iterable over ``datetime.date`` instances in the timeseries.'''
        c = self.dateinverse
        for key in self.keys(desc = desc):
            yield c(key)
            
    def keys(self, desc = None):
        '''Returns an iterable over ``raw`` keys. The keys may be different from dates
for same backend implementations.'''
        raise NotImplementedError
            
    def values(self, desc = None):
        '''Returns a ``numpy.ndarray`` containing the values of the timeseries.
Implementations should try not to copy data if possible. This function
can be used to access the timeseries as if it was a matrix.'''
        raise NotImplementedError
    
    def items(self, desc = None):
        '''Returns a python ``generator`` which can be used to iterate over
:func:`dynts.TimeSeries.dates` and :func:`dynts.TimeSeries.values` returning a two dimensional
tuple ``(date,value)`` in each iteration. Similar to the python dictionary items
function. The additional input parameter *desc* can be used to iterate from
the greatest to the smallest date in the timeseries by passing ''desc=True``'''
        for d,v in izip(self.dates(desc = desc),self.values(desc = desc)):
            yield d,v
            
    def series(self):
        data = self.values()
        for c in range(self.count()):
            yield data[:,c]
            
    def serie(self, index):
        return self.values()[:,index]
    
    def display(self):
        for d,v in self.items():
            print('%s: %s' % (d,v))
    
    def isconsistent(self):
        '''Check if the timeseries is consistent'''
        for dt1,dt0 in laggeddates(self):
            if dt1 <= dt0:
                return False
        return True
    
    # PURE VIRTUAL FUNCTIONS
    
    def start(self):
        '''Start date of timeseries'''
        raise NotImplementedError
    
    def end(self):
        '''End date of timeseries'''
        raise NotImplementedError
    
    def frequency(self):
        '''Average frequency of dates'''
        raise NotImplementedError
    
    @property
    def shape(self):
        raise NotImplementedError
    
    def __getitem__(self, i):
        raise NotImplementedError
    
    def colnames(self):
        raise NotImplementedError
    
    def delta(self, k = 1):
        raise NotImplementedError
    
    def lag(self, k = 1):
        raise NotImplementedError
    
    def log(self):
        raise NotImplementedError
    
    def logdelta(self):
        pass
    
    def stddev(self):
        raise NotImplementedError
    
    def apply(self, func, **kwargs):
        return self._rollapply(func, window = len(self), **kwargs)
    
    def rollapply(self, func, window = 20,
                  align = 'right', bycolumn = True,  **kwargs):
        '''A generic :ref:`rolling function <rolling-function>` for function *func*.

* *func* string indicating function, such as ``min``, ``max``, ``std`` and so forth
* *window* number of point per group.
* *align* string specifying whether the index of the result should be ``left`` or
  ``right`` (default) or ``centered`` aligned compared to the rolling window of observations.
* *bycolumn* if ``True`` each *func* will be applied to each column separately.'''
        window = window or len(self)
        self.precondition(window<=len(self) and window > 0,DyntsOutOfBound)
        return self._rollapply(func, window = window,
                               align = align,
                               bycolumn = bycolumn, **kwargs)
    
    def rollmax(self, **kwargs):
        '''A :ref:`rolling function <rolling-function>` for max values'''
        return self.rollapply('max',**kwargs)
    
    def rollmin(self, **kwargs):
        '''A :ref:`rolling function <rolling-function>` for max values'''
        return self.rollapply('min',**kwargs)
    
    def rollmedian(self, **kwargs):
        '''A :ref:`rolling function <rolling-function>` for median values'''
        return self.rollapply('median',**kwargs)
    
    def rollmean(self, **kwargs):
        '''A :ref:`rolling function <rolling-function>` for mean values'''
        return self.rollapply('mean',**kwargs)
    
    def window(self, start, end):
        raise NotImplementedError
    
    def merge(self, ts, all = True):
        raise NotImplementedError
    
    def clone(self, date = None, data = None, name = None):
        name = name or self.name
        data = data if data is not None else self.values()
        ts = self.__class__(name)
        if date is None:
            ts.make(self.keys(),data,raw=True)
        else:
            ts.make(date,data)
        return ts
        
    def __add__(self, other):
        return addts(self,other)
    
    # INTERNALS
    ################################################################
    
    def _rollapply(func, window = 20, **kwargs):
        raise NotImplementedError
    
    def make(self, date, data, **kwargs):
        '''Internal function to create the inner data:
        
* *date* iterable/iterator/generator over dates
* *data* iterable/iterator/generator over values'''
        raise NotImplementedError
    
    def precondition(self, precond, errorclass = DyntsException, msg = ''):
        if not precond:
            raise errorclass(msg)


import unittest
from itertools import izip

import dynts
from dynts.utils import skiplist, rollingOperation
from dynts.utils.populate import populate

class RollingFunctionSkipList(unittest.TestCase):
    
    def testSkipList(self):
        data  = populate(size = 500)[:,0]
        ol = skiplist(data = data)
        reduce(lambda x,y: self.assertTrue(y>x),ol)
        
    def testRollingOp(self):
        data  = populate(size = 500)[:,0]
        roll  = rollingOperation(data,20)
        rmin  = roll.min()
        rmax  = roll.max()
        rmed  = roll.median()
        for m0,m1,m2 in izip(rmin,rmed,rmax):
            self.assertTrue(m1>=m0)
            self.assertTrue(m2>=m1)
        
        


from dynts.dsl.grammar import *
from dynts.conf import settings
from dynts.exceptions import DyntsException
from dynts.backends import istimeseries, isxy
from dynts.dsl.registry import FunctionBase, function_registry
from dynts.utils import smart_str


def parse(timeseries_expression, method = None, functions = None):
    '''Function for parsing :ref:`timeseries expressions <dsl-script>`.
If succesful, it returns an instance of :class:`dynts.dsl.Expr`.'''
    from ply import yacc
    from rules import rules
    functions = functions if functions is not None else function_registry
    ru = rules(functions)
    ru.build()
    ru.input(smart_str(timeseries_expression).lower())
    tokens     = ru.tokens
    precedence = ru.precedence
    yacc       = yacc.yacc(method = method or 'SLR')
    return yacc.parse(lexer = ru.lexer)


def merge(series):
    '''Merge timeseries. *series* must be an iterable over
timeseries.'''
    series = iter(series)
    ts = series.next()
    return ts.merge(series)

    
class dslresult(object):
    
    def __init__(self, expression, data, backend = None):
        self.expression = expression
        self.data = data
        self.backend = backend or settings.backend
        
    def __unicode__(self):
        return u'%s' % self.expression
    
    def __repr__(self):
        return self.expression.__repr__()
    
    def __str__(self):
        return self.__repr__()
    
    def unwind(self):
        if not hasattr(self,'_ts'):
            self._unwind()
        return self
    
    def ts(self):
        self.unwind()
        return self._ts
    
    def xy(self):
        self.unwind()
        return self._xy
        
    def _unwind(self):
        res = self.expression.unwind(self.data,self.backend)
        self._ts = None
        self._xy = None
        if istimeseries(res):
            self._ts = res
        elif res and isinstance(res,list):
            tss = []
            xys = []
            for v in res:
                if istimeseries(v):
                    tss.append(v)
                elif isxy(v):
                    xys.append(v)
            if tss:
                self._ts = merge(tss)
            if xys:
                self._xy = xys
        elif isxy(res):
            self._xy = res
            
    def dump(self, format, **kwargs):
        ts = self.ts()
        xy = self.xy()
        if istimeseries(ts):
            ts = ts.dump(format, **kwargs)
        else:
            ts = None
        if xy:
            if isxy(xy):
                xy = [xy]
            for el in xy:
                ts = el.dump(format, container = ts, **kwargs)
        return ts
            
        



from math import log,sqrt

def laggeddates(ts, step = 1):
    '''Lagged iterator over dates'''
    if step == 1:
        dates = ts.dates()
        if not hasattr(dates,'next'):
            dates = dates.__iter__()
        dt0 = dates.next()
        for dt1 in dates:
            yield dt1,dt0
            dt0 = dt1
    else:
        q = deque()
        while done:
            done+=1
            lag.append(dates.next())
        for dt1 in dates:
            lag.append(dt1)
            yield dt1,lag.pop(0)
            
                
def laggeditems(ts, step = 1):
    '''Iteration over lagged items.'''
    if setp == 1:
        items = self.items()
        d0,i0 = items.next()
        for d1,i1 in dates:
            yield d1,i1,d0,i0
            d0 = d1
            i0 = i1
    else:
        ts.precondition(step > 1 and step < len(ts), dynts.DyntsOutOfBound) 
        done = 0
        items  = self.items()
        lag    = deque()
        while done:
            done+=1
            lag.append(items.next())
        for item1 in items:
            lag.append(item1)
            yield item1+lag.pop(0)
            
            
def logdeltadt(ts, step = 1, dcf = None):
    '''Iterator which returns a log delta'''
    dcf = dcf or settings.getdc()
    for d1,i1,d0,i0 in laggeditems(ts,step):
        dt = dcf(d0,d1)
        yield d1,log(i1/i0)/sqrt(dt)
    




def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
    """Returns a bytestring version of 's', encoded as specified in 'encoding'.
If strings_only is True, don't convert (some) non-string-like objects.
"""
    if strings_only and isinstance(s, (types.NoneType, int)):
        return s
    if not isinstance(s, basestring):
        try:
            return str(s)
        except UnicodeEncodeError:
            if isinstance(s, Exception):
                # An Exception subclass containing non-ASCII data that doesn't
                # know how to print itself properly. We shouldn't raise a
                # further exception.
                return ' '.join([smart_str(arg, encoding, strings_only,
                        errors) for arg in s])
            return unicode(s).encode(encoding, errors)
    elif isinstance(s, unicode):
        return s.encode(encoding, errors)
    elif s and encoding != 'utf-8':
        return s.decode('utf-8', errors).encode(encoding, errors)
    else:
        return s


import unittest
import numpy as np
import random
from dynts.utils.populate import populate
from dynts.utils.populate import datepopulate
from dynts.backends.dev import TSWithOperators as TimeSeries

__all__ = ['TestOperators',
           ]

add = lambda x, y : x + y
sub = lambda x, y : x - y
mul = lambda x, y : x * y
div = lambda x, y : x / y

def isiterable(a):
    return hasattr(a,'__iter__')

class CommonTSUtils(unittest.TestCase):

    def getdata(self, size = 100, cols = 1,delta = 1):
        date = datepopulate(size = size, delta = delta)
        data = populate(size = size, cols = cols)
        return date,data

    def getts(self, returndata = False, cols = 1, delta = 1):
        date,data = self.getdata(100,cols = cols,delta = delta)

        ts   = TimeSeries(name = 'test', date = date, data = data)
        if returndata:
            return ts,list(date),list(data)
        else:
            return ts
        
    def assertAlmostEqual(self, a, b): #copied from pandas
        if isiterable(a):
            np.testing.assert_(isiterable(b))
            np.testing.assert_equal(len(a), len(b))
            for i in xrange(len(a)):
                self.assertAlmostEqual(a[i], b[i])
            return True

        err_msg = lambda a, b: 'expected %.5f but got %.5f' % (a, b)

        if np.isnan(a):
            np.testing.assert_(np.isnan(b))
            return

        # case for zero
        if abs(a) < 1e-5:
            np.testing.assert_almost_equal(a, b, decimal=5, err_msg=err_msg(a, b), verbose=False)
        else:
            np.testing.assert_almost_equal(1, a/b, decimal=5, err_msg=err_msg(a, b), verbose=False)

#    def assertAlmostEqual(self, exp, recv, tol = 2, msg = None):
#        if msg is None:
#            msg = "Values do not match expected %s, received %s" %(exp, recv)
#        unittest.TestCase.assertAlmostEqual(self, exp,recv, tol, msg)

    def assertEqual(self, exp, recv, msg = None):
        if msg is None:
            msg = "Values do not match expected %s, received %s" %(exp, recv)
        unittest.TestCase.assertEqual(self, exp,recv,msg)

    def check_dates(self, ts, dts):
        ts_dts = list(ts.dates())
        self._check_vectors(ts_dts, dts, equal=True)
        
    def check_values(self, ts, vals):
        ts_vals = list(ts.values())
        self._check_vectors(ts_vals, vals, equal=False)

    def _check_vectors(self, v1, v2, equal=True):
        lv1 = len(v1)
        lv2 = len(v2)
        self.assertEqual(lv1, lv2, "Vectors are of different lengths %s, %s" %(lv1, lv2))

        for a,b  in zip(v1,v2):
            if equal:
                self.assertEqual(a, b)
            else:
                self.assertAlmostEqual(a,b)

class TestOperators(CommonTSUtils):

    ops = (add, sub, mul, div)
    def testTSArithOperators(self):
        ops = self.ops
        ts1, dates1, data1 = self.getts(returndata = True, cols = 2)
        ts2, dates2, data2 = self.getts(returndata = True, cols = 2)
        for op in ops:
           # print op
            ts3 = op(ts1,ts2)
            exp = map(op, data1, data2)
            self.check_dates(ts3, dates1) #the dates should be the same
            self.check_values(ts3, exp) #the values should be 

    def testScalarArithOperators(self):
        ops = self.ops
        delta = random.uniform(1.0,100.0)
        ts, dates, data = self.getts(returndata = True)
        for op in ops:
            new_ts = op(ts, delta)
            curry_op = lambda x : op(x, delta)

            exp = map(curry_op, data)
            self.check_dates(new_ts, dates)
            self.check_values(new_ts, exp)




import unittest
from dynts.utils import cross


class CrossectionTest(unittest.TestCase):
    
    def testAllGE(self):
        self.assertTrue(cross([1,2]) >= [-1,0.5])
        self.assertTrue(cross([1,2,78]) >= [-1,1,56])
        self.assertTrue(cross([1,2,56]) >= [-1,1,56])
        self.assertFalse(cross([1,2,-5]) >= [-1,1,56])
        
    def testAllLE(self):
        self.assertFalse(cross([1,2]) <= [-1,1])
        self.assertFalse(cross([1,2,78]) <= [-1,1,56])
        self.assertFalse(cross([1,2,56]) <= [-1,1,56])
        self.assertTrue(cross([1,2,-5]) <= [4,2,56])
        self.assertTrue(cross([-90,1,-5]) <= [-1,1,56])
    



class XigniteProxy(object):
    baseurl  = 'http://www.xignite.com/'
    products = {'ccy':'',
                'rates':'xRates.asmx'}

    

class xignite(DataProvider):
    proxy   = XigniteProxy()
    baseurl = 'http://www.xignite.com/xCurrencies.asmx/'
    
    def get(self, ticker, startdate, enddate, field = None):
        
    


from bisect import bisect_left, bisect_right

from dynts.conf import settings
from dynts.utils.section import asarray
from dynts.exceptions import *

__all__ = ['TimeSerieWrap','asbtree','ashash']


class TimeSerieWrap(object):
    
    def __init__(self, ts, **kwargs):
        self.ts = ts
        self.modified = False
        self.names = ts.names()
        self.wrap(**kwargs)
        
    def wrap(self, **kwargs):
        pass
        
    def get(self, dt, default = None):
        '''Equivalent to::

    self[dt]
    
but catches exceptions and return *default*.'''
        try:
            return self[dt]
        except DateNotFound:
            return default
        
    def getts(self):
        return self.ts
        
    @property
    def shape(self):
        return (len(self),self.ts.count())


class asbtree(TimeSerieWrap):
    '''Wrap a :class:`dynts.TimeSeries` and
expose binary-tree like functionalities. A :class:`dynts.TimeSeries` instance
has a shortcut method which construct a ``asbtree``. Here is an example::

    >>> from dynts.utils.populate import randomts, date
    >>> ts = randomts(cols = 2, start = date(2010,1,1), size = 50)
    >>> dts = ts.asbtree()
    >>> sts.find_ge(ts.start())
    1
    >>> sts.find_ge(ts.end())
    49
'''
    def wrap(self):
        ts = self.ts
        self.dates  = asarray(ts.dates())
        self.values = ts.values()
    
    def __len__(self):
        return len(self.dates)
    
    def __getitem__(self, dt):
        '''Get the value at *dt* otherwise it raises
an :class:`dynts.exceptions.DateNotFound`.'''
        try:
            index = self.find_ge(dt)
        except DyntsOutOfBound:
            raise DateNotFound
        if self.dates[index] == dt:
            return self.values[index]
        else:
            raise DateNotFound
        
    def find_ge(self, dt):
        '''Building block of all searches. Find the index
corresponding to the leftmost value greater or equal to *dt*.
If *dt* is greater than the
:func:`dynts.TimeSeries.end` a :class:`dynts.exceptions.RightOutOfBound`
exception will raise.

*dt* must be a python datetime.date instance.'''
        i = bisect_left(self.dates,dt)
        if i != len(self.dates):
            return i
        raise RightOutOfBound
    
    def find_le(self, dt):
        '''Find the index corresponding to the rightmost
value less than or equal to *dt*.
If *dt* is less than :func:`dynts.TimeSeries.end`
a :class:`dynts.exceptions.LeftOutOfBound`
exception will raise.

*dt* must be a python datetime.date instance.'''
        i = bisect_right(self.dates,dt)
        if i:
            return i-1
        raise LeftOutOfBound

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__,self.ts.__repr__())
    
    def __str__(self):
        return '%s(%s)' % (self.__class__.__name__,self.ts)


class ashash(TimeSerieWrap):
    
    def wrap(self):
        ts = self.ts
        hash = {}
        self.hash   = hash
        self.keys   = hash.iterkeys
        self.values = hash.itervalues
        for dt,v in ts.items():
            hash[dt] = v
    
    def __len__(self):
        return len(self.hash)
        
    def __getitem__(self, dt):
        '''Get the value at *dt* otherwise it raises
an :class:`dynts.exceptions.DateNotFound`.'''
        try:
            return self.hash[dt]
        except KeyError:
            raise DateNotFound
    
    def __setitem__(self, dt, item):
        self.hash[dt] = item
        self.modified = True

    def items(self):
        h = self.hash
        for dt in sorted(self.keys()):
            yield dt,h[dt]
            
    def getts(self):
        if self.modified:
            name   = settings.splittingnames.join(self.names)
            hash   = self.hash
            dates  = sorted(self.keys())
            values = (hash[dt] for dt in dates)
            return self.ts.clone(name = name, date = dates, data = values)
        else:
            return self.ts
    


'''Quantitative financial timeseries analysis'''

VERSION = (0, 3, 1)
 
def get_version():
    if len(VERSION) == 3:
            v = '%s.%s.%s' % VERSION
    else:
        v = '%s.%s' % VERSION[:2]
    return v
 
__version__  = get_version()
__license__  = "BSD"
__author__   = "Luca Sbardella"
__contact__  = "luca@quantmind.com"
__homepage__ = "http://github.com/quantmind/dynts/"


from dynts.exceptions import *
from backends import timeseries, xydata, TimeSeries, DynData
from backends import istimeseries, Formatters, BACKENDS
from dsl import parse, merge, dslresult, function_registry, functions
from data import providers
import formatters
Formatters['flot'] = formatters.ToFlot()
Formatters['jsonvba'] = formatters.ToJsonVba()
Formatters['csv']  = formatters.ToCsv()
Formatters['xls']  = formatters.ToXls()
Formatters['plot'] = formatters.ToPlot()


def evaluate(expression, start = None, end = None,
             loader = None, logger = None, backend = None, **kwargs):
    '''Evaluate expression *e*. This and :func:`dynts.parse`
represent the main entry point of the library.
    
* *expression* string or an instance of :class:`dynts.dsl.Expr` obtained using
  the :func:`dynts.parse` function.
* *start* start date or ``None``.
* *end* end date or ``None``.
* *loader* Optional :class:`dynts.data.TimeSerieLoader` class or instance.
* *logger* Python logging class or ``None``. Used if you required logging.
* *backend* :class:`dynts.TimeSeries` backend name or ``None``.

*expression* is parsed and the :class:`dynts.expr.Symbol` are sent to the
:class:`dynts.data.TimeSerieLoader` instance for retrieving actual timeseries data.
It returns an instance of :class:`dynts.dslresult`.

Typical usage::

    >>> import dynts
    >>> r = dynts.evaluate('min(GS,window=30)')
    >>> r
    min(GS,window=30)
    >>> ts = r.unwind()
    '''
    if isinstance(expression,basestring):
        expression = parse(expression)
    symbols = expression.symbols()
    data = providers.load(symbols, start, end, loader = loader,
                          logger = logger, backend = backend, **kwargs)
    return dslresult(expression, data, backend = backend)


def tsname(*names):
    from dynts.conf import settings
    return reduce(lambda x,y: '%s%s%s' % (x,settings.splittingnames,y), names)


################### For testings
#    
def add2path():
    import os
    import sys
    path = os.path.split(os.path.split(os.path.abspath(__file__))[0])[0]
    if path not in sys.path:
        sys.path.insert(0,path)
        
def runtests():
    import unittest
    add2path()
    from dynts import tests
    loader = unittest.TestLoader()
    suite  = loader.loadTestsFromModule(tests)
    runner = unittest.TextTestRunner()
    runner.run(suite)
    
def runbench():
    add2path()
    from dynts.bench import runbench
    runbench()


from dynts import xydata
from dynts.exceptions import FunctionTypeError, DateNotFound
from dynts.dsl import FunctionBase
from dynts.conf import settings


class SimpleFunctionXY(FunctionBase):
    abstract = True
    def __call__(self, args, **kwargs):
        return self.apply(args, **kwargs)
        

class scatter(SimpleFunctionXY):
    """A simple scatter plot"""
    def apply(self, args, **kwargs):
        if not len(args) == 2:
            raise FunctionTypeError(self,"function requires two timeseries")
        ismissing = settings.ismissing
        ts0 = args[0]
        ts1 = args[1].ashash();
        name = '%s(%s,%s)' % (self.name,ts0.name,args[1].name)
        data = []
        for dt,v0 in ts0.items():
            try:
                v1 = ts1[dt]
            except DateNotFound:
                continue
            data.append([v0[0],v1[0]])
        return xydata(name = name, data = data, lines = False)
    
    


from dynts.dsl.ast.binmath import *

class Function(Bracket):
    '''A :class:`dynts.dsl.Bracket` representing a function *func*, an instance of :class:`dynts.dsl.FunctionBase`.
A function is defined a-la Python as::
    
    func(expression, **kwargs)
    
where *kwargs* is a dictionary of input parameters. For example, the rolling-standard deviation
is defined as::

    std(expression,window=20)
    '''
    
    def __init__(self, func, expression, pl, pr):
        self.func = func
        super(Function,self).__init__(expression, pl, pr)
            
    def info(self):
        return '%s%s' % (self.func,super(Function,self).info())
    
    def _unwind(self, values, backend, **kwargs):
        args,kwargs = super(Function,self)._unwind(values, backend, **kwargs)
        return self.func(args,**kwargs)




__all__ = ['isnumeric']

def isnumeric(obj):
    '''
    Return true if obj is a numeric value
    '''
    from decimal import Decimal
    if type(obj) == Decimal:
        return True
    else:
        try:
            float(obj)
        except:
            return False
        return True


from rpy2 import rinterface
import numpy as ny

import dynts
from dynts.utils import ascolumn
from dynts.utils.rutils import rpyobject, py2rdate, r2pydate, isoformat



class rts(dynts.TimeSeries,rpyobject):
    '''Base class for R-based timeseries objects'''
    
    @property
    def shape(self):
        try:
            s = tuple(self.rc('dim'))
        except:
            s = self.values().shape
        if len(s) == 1:
            s += 1,
        return s
    
    def __getitem__(self, i):
        '''This is not an efficient method'''
        return self.values()[i]
    
    def factory(self, date, data, raw = False):
        raise NotImplementedError
    
    def dateconvert(self, dte):
        return py2rdate(dte)
    
    def dateinverse(self, key):
        return r2pydate(key)
    
    def make(self, date, data, raw = False):
        if date is None:
            ts = None
        else:
            data = ascolumn(data)
            ts = self.factory(date, data, raw = raw)
        self._ts = ts
        
    def keys(self, desc = None):
        '''numpy asarray does not copy data'''
        res = ny.asarray(self.rc('index'))
        if desc == True:
            return reversed(res)
        else:
            return res
        
    def values(self, desc = None):
        '''numpy asarray does not copy data'''
        if self._ts:
            res = ny.asarray(self._ts)
            if desc == True:
                return reversed(res)
            else:
                return res
        else:
            return ny.ndarray([0,0])
        
    def lag(self, k = 1):
        return self.rcts('lag',k)
    
    def delta(self, k = 1):
        return self.rcts('diff',k)
    
    def log(self):
        return self.rcts('log')
    
    def stddev(self):
        raise self.rcts('sd')
    
    def isregular(self):
        return self.rc('is.regular')[0]
    
    def frequency(self):
        return self.rc('frequency')[0]
    
    def _rollapply(self, func, window = None, **kwargs):
        return self.rcts('roll%s' % func, window, **kwargs)
    
    def window(self, start, end):
        c = self.dateconvert
        return self.rcts('window', start = c(start), end = c(end))

    def rc(self, command, *args, **kwargs):
        return self.r[command](self._ts,*args,**kwargs)
    
    def rcts(self, command, *args, **kwargs):
        cls = self.__class__
        name = kwargs.pop('name','')
        date = kwargs.pop('date',None)
        data = kwargs.pop('data',None)
        kwargs.pop('bycolumn',None)
        ts  = cls(name=name,date=date,data=data)
        ts._ts = self.rc(command, *args, **kwargs)
        return ts



benchmarks = []
import timeit
#import cProfile as profiler


def run_timer(name, number, repeat, timer):
    v = min(timer.repeat(repeat=repeat,number=number))/number
    print('%s = %s' % (name,v))


test_maker = lambda name,timer : lambda number,repeat : run_timer(name, number, repeat, timer)


#def profile(name):
#    profiler.run("from unuk.http import tests\ntests.testers[%s]('%s')" % (n,url))



def makebench(name, **kwargs):
    '''Create a benchmark and added it to the list of benchmarks.'''
    global benchmarks
    timer = timeit.Timer(**kwargs)
    benchmarks.append(test_maker(name,timer))
    
    
    
def runbench(number = 5, repeat = 5):
    global benchmarks
    for b in benchmarks:
        b(number,repeat)
        


import os
import sys

def _resolve_name(name, package, level):
    """Return the absolute name of the module to be imported."""
    if not hasattr(package, 'rindex'):
        raise ValueError("'package' not set to a string")
    dot = len(package)
    for x in xrange(level, 1, -1):
        try:
            dot = package.rindex('.', 0, dot)
        except ValueError:
            raise ValueError("attempted relative import beyond top-level "
                              "package")
    return "%s.%s" % (package[:dot], name)

def import_module(name, package=None):
    """Import a module.

The 'package' argument is required when performing a relative import. It
specifies the package to use as the anchor point from which to resolve the
relative import to an absolute import.

"""
    if name.startswith('.'):
        if not package:
            raise TypeError("relative imports require the 'package' argument")
        level = 0
        for character in name:
            if character != '.':
                break
            level += 1
        name = _resolve_name(name[level:], package, level)
    __import__(name)
    return sys.modules[name]


def add_to_path(path):
    if os.path.isdir(path):
        if path not in sys.path:
            sys.path.insert(0, path)
        return path
    else:
        raise ValueError("Path %s is not a valid directory" % path)
    
    
def load_scripts(path):
    '''Load scripts from a directory into a dictionary'''
    scripts = {}
    if not os.path.isdir(path):
        return scripts
    files = os.listdir(path)
    for f in files:
        if f.startswith('__'):
            continue
        fullpath = os.path.join(path, f)
        if os.path.isfile(fullpath):
            sf = f.split('.')
            if len(sf) == 2 and sf[1] == 'py':
                file = open(fullpath,'r')
                scripts[sf[0]] = file.read()
                file.close()
    return scripts




# Original from
# http://code.activestate.com/recipes/576930-efficient-running-median-using-an-indexable-skipli/
#
import math
from random import random
from math import log, ceil
from collections import deque
from itertools import islice

__all__ = ['skiplist','rollingOperation']

class Node(object):
    __slots__ = 'value', 'next', 'width'
    def __init__(self, value, next, width):
        self.value, self.next, self.width = value, next, width

class End(object):
    'Sentinel object that always compares greater than another object'
    def __cmp__(self, other):
        return 1

NIL = Node(End(), [], [])               # Singleton terminator node


class skiplist:
    'Sorted collection supporting O(lg n) insertion, removal, and lookup by rank.'

    def __init__(self, expected_size=100, data = None):
        if data is not None:
            if hasattr(data,'__len__'):
                expected_size = max(expected_size,len(data))
        self.size = 0
        self.maxlevels = int(1 + log(expected_size, 2))
        self.head = Node('HEAD', [NIL]*self.maxlevels, [1]*self.maxlevels)
        if data is not None:
            insert = self.insert
            for value in data:
                insert(value)

    def __len__(self):
        return self.size

    def __getitem__(self, i):
        node = self.head
        i += 1
        for level in reversed(range(self.maxlevels)):
            while node.width[level] <= i:
                i -= node.width[level]
                node = node.next[level]
        return node.value

    def insert(self, value):
        # find first node on each level where node.next[levels].value > value
        chain = [None] * self.maxlevels
        steps_at_level = [0] * self.maxlevels
        node = self.head
        for level in reversed(range(self.maxlevels)):
            while node.next[level].value <= value:
                steps_at_level[level] += node.width[level]
                node = node.next[level]
            chain[level] = node

        # insert a link to the newnode at each level
        d = min(self.maxlevels, 1 - int(log(random(), 2.0)))
        newnode = Node(value, [None]*d, [None]*d)
        steps = 0
        for level in range(d):
            prevnode = chain[level]
            newnode.next[level] = prevnode.next[level]
            prevnode.next[level] = newnode
            newnode.width[level] = prevnode.width[level] - steps
            prevnode.width[level] = steps + 1
            steps += steps_at_level[level]
        for level in range(d, self.maxlevels):
            chain[level].width[level] += 1
        self.size += 1

    def remove(self, value):
        # find first node on each level where node.next[levels].value >= value
        chain = [None] * self.maxlevels
        node = self.head
        for level in reversed(range(self.maxlevels)):
            while node.next[level].value < value:
                node = node.next[level]
            chain[level] = node
        if value != chain[0].next[0].value:
            raise KeyError('Not Found')

        # remove one link at each level
        d = len(chain[0].next[0].next)
        for level in range(d):
            prevnode = chain[level]
            prevnode.width[level] += prevnode.next[level].width[level] - 1
            prevnode.next[level] = prevnode.next[level].next[level]
        for level in range(d, self.maxlevels):
            chain[level].width[level] -= 1
        self.size -= 1

    def __iter__(self):
        'Iterate over values in sorted order'
        node = self.head.next[0]
        while node is not NIL:
            yield node.value
            node = node.next[0]


def smax(olist,missing):
    if olist:
        return olist[len(olist)-1]
    else:
        return missing

def smin(olist,missing):
    if olist:
        return olist[0]
    else:
        return missing

def smedian(olist,missing):
    if olist:
        midpoint = len(olist) // 2
        return olist[midpoint]
    else:
        return missing



class rollingOperation(object):
    
    def __init__(self, iterable, window):
        from dynts.conf import settings
        self.iterable = iterable
        self.window = window
        self.missing = settings.missing_value
        self.ismissing = settings.ismissing
        
    def min(self):
        return self.rolling(smin)
    
    def max(self):
        return self.rolling(smax)
    
    def median(self):
        return self.rolling(smedian)
    
    def rolling(self, op):
        'Fast rolling operation with O(log n) updates where n is the window size'
        missing   = self.missing
        ismissing = self.ismissing
        window = self.window
        it = iter(self.iterable)
        queue = deque(islice(it, window))
        ol    = skiplist(window)
        for elem in queue:
            if not ismissing(elem):
                ol.insert(elem)
        yield op(ol,missing)
        for newelem in it:
            oldelem = queue.popleft()
            if not ismissing(oldelem):
                ol.remove(oldelem)
            queue.append(newelem)
            if not ismissing(newelem):
                ol.insert(newelem)
            yield op(ol,missing)



from numpy import asarray
from rpy2.robjects import StrVector

from dynts.backends.rbase import rts, r2pydate, isoformat, ny


class TimeSeries(rts):
    '''Timeserie backend based on Rmetrics timeserie__ package
    
__ http://cran.r-project.org/web/packages/timeSeries/index.html
    '''
    type = 'rmetrics'
    libraries = ['timeSeries','zoo']
        
    def factory(self, date, data):
        tdate = self.dateconvert
        adt = StrVector([tdate(dt) for dt in date])
        #data = FloatVector(data)
        return self.r['timeSeries'](data, adt)
    
    def dateconvert(self, dte):
        return isoformat(dte)
    
    def dateinverse(self, x):
        return r2pydate(self.r['as.double'](x)[0]) 
    
    def keys(self):
        '''numpy asarray does not copy data'''
        v = self.rc('time')
        return ny.asarray(v)
                          
    
    def start(self):
        return self.dateinverse(self.rc('start'))
        
    def end(self):
        return self.dateinverse(self.rc('end'))


from django import http

from djpcms.views import appview

from ccy import date2yyyymmdd, dateFromString


def date2yyyymmdd(dte):
    return dte.day + 100*(dte.month + 100*dte.year)


class TimeSeriesView(appview.AppView):
    '''```djpcms``` application view for retriving timeseries data as  JSON string.
Available as Ajax Get response.'''
    _methods      = ('get',)
    
    def get_response(self, djp):
        request = djp.request
        if not request.is_ajax():
            raise http.Http404
        sdata = self.econometric_data(djp.request, dict(request.GET.items()))
        return http.HttpResponse(sdata, mimetype='application/javascript')
    
    def get_object(self, code):
        '''Check if the code is an instance of a model.'''
        codes = code.split(':')
        if len(codes) == 2 and codes[0] == str(self.model._meta):
            try:
                return self.model.objects.get(id = int(codes[1]))
            except:
                return None
                
    def econometric_data(self, request, data):
        #Obtain the data
        cts    = data.get('command',None)
        start  = data.get('start',None)
        end    = data.get('end',None)
        period = data.get('period',None)
        object = self.get_object(cts)
        if object:
            cts = self.codeobject(object)
        if start:
            start = dateFromString(str(start))
        if end:
            end = dateFromString(str(end))
        try:
            return self.getdata(request,cts,start,end)
        except IOError, e:
            return ''
    
    def codeobject(self, object):
        return str(object)
    
    def getdata(self,request,expression,start,end):
        '''Pure virtual function which retrives the actual data.
It must return a JSON string.'''
        raise NotImplementedError
    
    


from jflow.lib import tablefunction
from data import qpqn

__all__ = ['qp','qn','calmar','calmarnorm']

qn = tablefunction('qn',qpqn.qn)
qp = tablefunction('qp',qpqn.qp)

def calmar(sharpe, T = 1.0):
    '''
    Calculate the Calmar ratio for a Weiner process
    
    @param sharpe:    Annualized Sharpe ratio
    @param T:         Time interval in years
    '''
    x = 0.5*T*sharpe*sharpe
    return x/qp(x)


def calmarnorm(sharpe, T, tau = 1.0):
    '''
    Multiplicator for normalizing calmar ratio to period tau
    '''
    return calmar(sharpe,tau)/calmar(sharpe,T)


import unittest
import random
from dynts.utils.populate import populate
from dynts.utils.populate import datepopulate
from othello.impl.tseries import TimeSeries

__all__ = ['DevTestTS',
           ]

class CommonTSUtils(unittest.TestCase):

    def getdata(self, size = 100, delta = 1):
        date = datepopulate(size = size, delta = delta)
        data = populate(size = size)
        return date,data

    def getts(self, returndata = False, delta = 1):
        date,data = self.getdata(100,delta)

        ts   = TimeSeries(name = 'test', ts = zip(date, data))
        if returndata:
            return ts,list(date),list(data)
        else:
            return ts

    def assertAlmostEqual(self, exp, recv, tol = 2, msg = None):
        if msg is None:
            msg = "Values do not match expected %s, received %s" %(exp, recv)
        unittest.TestCase.assertAlmostEqual(self, exp,recv, tol, msg)

class DevTestTS(CommonTSUtils):

    def xtestAddTS(self):
        ts1, _, data1 = self.getts(returndata = True)
        ts2, _, data2 = self.getts(returndata = True)
        ts3 = ts2 + ts1

        exp = sum(data1) + sum(data2)
        recv = sum(ts3.values())
        self.assertAlmostEqual(exp, recv)

    def xtestAddScalar(self):
        delta = random.uniform(1.0,100.0)
        ts, dates, data = self.getts(returndata = True)
        new_ts = ts + delta
        for dt, val in zip(dates, data):
            exp = val + delta
            recv = new_ts[dt]
            self.assertAlmostEqual(exp, recv)



from dynts.conf import settings
from dynts import timeseries, istimeseries
from dynts.exceptions import *

def isnumber(value):
    try:
        float(value)
        return True
    except:
        return False

class Expr(object):
    '''Base class for abstract syntax nodes
    '''    
    def count(self):
        '''Number of nodes'''
        return 1
    
    def __unicode__(self):
        return u'%s' % self.info()
    
    @property
    def type(self):
        return self.__class__.__name__.lower()
    
    def info(self):
        return ''
    
    def __repr__(self):
        return '%s' % self.info()
    
    def __str__(self):
        return self.__repr__()
    
    def __eq__(self, other):
        return isinstance(other, Expr) and str(other) == str(self)
    
    def symbols(self):
        '''Return a list of :class:`dynts.dsl.Symbol` instances.'''
        return None
    
    def variables(self):
        return None
    
    def removeduplicates(self, entries = None):
        return None
    
    def unwind(self, values, backend, **kwargs):
        '''Unwind expression by applying *values* to the abstract nodes'''
        if not hasattr(self, "_unwind_value"):
            self._unwind_value = self._unwind(values, backend, **kwargs)
        return self._unwind_value
    
    def _unwind(self, values, backend, **kwargs):
        raise NotImplementedError("Unwind method missing for %s" % self)
    

class BaseExpression(Expr):
    '''Base class for single-value expression
    '''
    def __init__(self, value):
        self.value = value
        
    def info(self):
        return str(self.value)
        
    
class Expression(BaseExpression):
    '''Base class for single expression
    '''    
    def symbols(self):
        return self.value.symbols()
    
    def removeduplicates(self, entries = None):
        if entries == None:
            entries = {}
        if isinstance(self.value,Symbol):
            c = entries.get(str(self.value), None)
            if c:
                ov = [self.value]
                self.value = c
                return ov
            else:
                entries[str(self.value)] = self.value
                return None
        else:
            return self.value.removeduplicates(entries = entries)
    
    
class Number(BaseExpression):
    '''
    A simple number.
    This expression is a constant numeric value
    '''
    def __init__(self,value):
        super(Number,self).__init__(value)
        
    def _unwind(self, values, backend, **kwargs):
        return self.value


class String(BaseExpression):
    '''
    A simple string.
    This expression is a constant numeric value
    '''
    def __init__(self,value):
        super(String,self).__init__(str(value))
        
    def _unwind(self, values, backend, **kwargs):
        return unwind.stringData(self.value)


class Parameter(BaseExpression):
    
    def __init__(self, value):
        value = str(value).lower()
        super(Parameter,self).__init__(value)
        

class Symbol(BaseExpression):
    '''Timeserie symbol. This expression is replaced by a timeserie value for the symbol
    '''
    def __init__(self, value, field = None):
        value = settings.symboltransform(value)
        super(Symbol,self).__init__(value)
    
    def symbols(self):
        return [str(self)]
    
    def _unwind(self, values, backend, **kwargs):
        sdata = values[str(self)]
        if istimeseries(sdata):
            return sdata
        else:
            ts = timeseries(name = str(self),
                            date = sdata['date'],
                            data = sdata['value'],
                            backend = backend)
            values[ts.name] = ts
            return ts
    
    def lineardecomp(self):
        return linearDecomp().append(self)


class MultiExpression(Expr):
    '''Base class for expression involving two or more elements
    '''
    def __init__(self, concat_operator, concatenate = True):
        self.__concatenate   = concatenate
        self.children        = []
        self.concat_operator = concat_operator
        
    def __len__(self):
        return len(self.children)
    
    def __iter__(self):
        return self.children.__iter__()
        
    def info(self):
        c = self.concat_operator
        return reduce(lambda x,y: '%s%s%s' % (x,c,y),self.children)
    
    def symbols(self):
        cs = []
        for c in self.children:
            ns = c.symbols()
            if ns:
                for n in ns:
                    if n not in cs:
                        cs.append(n)
        return cs
    
    def append(self, el):
        if isinstance(el,self.__class__) and self.__concatenate:
            for c in el:
                self.append(c)
        elif isinstance(el,Expr):
            self.children.append(el)
        else:
            raise ValueError("%s is not a valid grammar expression" % el)
        return el
            
    def __getitem__(self, idx):
        return self.children[idx]
    
    def removeduplicates(self, entries = None):
        '''
        Loop over children a remove duplicate entries.
        @return - a list of removed entries
        '''
        removed      = []
        if entries == None:
            entries = {}
        new_children = []
        for c in self.children:
            cs = str(c)
            cp = entries.get(cs,None)
            if cp:
                new_children.append(cp)
                removed.append(c)
            else:
                dups = c.removeduplicates(entries)
                if dups:
                    removed.extend(dups)
                entries[cs] = c
                new_children.append(c)
        self.children = new_children
        return removed
            
            

class ConcatOp(MultiExpression):
    '''
    Refinement of MultiExpression with a new constructor.
    This class simply define a new __init__ method
    '''
    def __init__(self, left, right, op, concatenate = True):
        super(ConcatOp,self).__init__(op, concatenate = concatenate)
        self.append(left)
        self.append(right)
        


class ConcatenationOp(ConcatOp):
    
    def __init__(self,left,right):
        super(ConcatenationOp,self).__init__(left, right, settings.concat_operator)
        
    def _unwind(self, values, backend, sametype = True, **kwargs):
        result = []
        for child in self:
            result.append(child.unwind(values, backend))
        return result
    
    
class SplittingOp(ConcatOp):
    
    def __init__(self,left,right):
        super(SplittingOp,self).__init__(left, right, settings.separator_operator)
        
    def _unwind(self, values, backend, **kwargs):
        ts = unwind.listData(label = str(self))            
        for c in self:
            v = c.unwind(values, backend)
            ts.append(v)
        return ts

        
class BinOp(ConcatOp):
    
    def __init__(self,left,right,op):
        if op in settings.special_operators:
            raise ValueError('Conactentaion operator "%s" is not a valid binary operator' % op)
        super(BinOp,self).__init__(left, right, op, concatenate = False)
        self.append = None
        
    def __get_left(self):
        return self[0]
    left = property(fget = __get_left)
    
    def __get_right(self):
        return self[1]
    right = property(fget = __get_right)
    
    
class EqualOp(BinOp):
    '''Equal operator expression. For example
    
* ``window = 35``
* ``param = AMZN``

The left hand side is the name of the parameter, while the right-hand side
is the parameter values which can be a :class:`Symbol`.
The left hand side is **never** a symbol. 
'''
    def __init__(self,left,right):
        if not isinstance(left,Parameter):
            if not isinstance(left,Symbol):
                raise ValueError('Left-hand-side of %s should be a string' % self)
            left = Parameter(left.value)
        super(EqualOp,self).__init__(left,right,"=")
        
    def _unwind(self, values, backend, **kwargs):
        data = self.right.unwind(values, backend, **kwargs)
        return {str(self.left):data}
 
 
class Bracket(Expression):
    '''A :class:`dynts.dsl.Expr` class for enclosing group of :class:`dynts.dsl.Expr`.
It forms the building block of :class:`dynts.dsl.Function` and other operators.'''
    def __init__(self,value,pl,pr):
        self.__pl = pl
        self.__pr = pr
        super(Bracket,self).__init__(value)
    
    def info(self):
        return '%s%s%s' % (self.__pl,self.value,self.__pr)
    
    def _unwind(self, *args, **kwargs):
        data = self.value.unwind(*args, **kwargs)
        if not isinstance(data,list):
            data = [data]
        args = []
        kwargs = {}
        for item in data:
            if isinstance(item,dict):
                kwargs.update(item)
            else:
                args.append(item)
        return args,kwargs
                
                
class uMinus(Expression):
    def __init__(self,value):
        Expression.__init__(self,value)
        
    def info(self):
        return '-%s' % self.value
    
    def lineardecomp(self):
        return linearDecomp().append(self,-1)
    


from django.conf import settings
from django import forms
from django.template import loader

from djpcms.utils import mark_safe
from djpcms.plugins import DJPplugin
from djpcms.views import appsite

skin = getattr(settings,'DYNTS_SKIN','dynts/ecoplot/skins/smooth.css')


class EcoForm(forms.Form):
    url    = forms.CharField()
    height = forms.IntegerField()
    
class EcoPlot(DJPplugin):
    '''Plugin for djpcms__ content management system.

__ http://packages.python.org/djpcms/'''
    name = "econometric-plot"
    description = "Econometric plot"
    form = EcoForm
    
    class Media:
        css = {
            'all': ('dynts/ecoplot/ecoplot.css', skin)
        }
        js = ['dynts/flot/excanvas.min.js',
              'dynts/flot/jquery.flot.min.js',
              'dynts/flot/jquery.flot.selection.min.js',
              'dynts/ecoplot/ecoplot.js',
              'dynts/decorator.js']
    
    def render(self, djp, wrapper, prefix, height = 400, url = '', **kwargs):
        height = abs(int(height))
        ctx = {'url':    url,
               'height': height,
               'item':   djp.instancecode()}
        return loader.render_to_string('dynts/econometric-plot.html', ctx)
    

class EconometricFunctions(DJPplugin):
    name = "econometric-functions"
    description = "Econometric Function"
    
    def render(self, djp, wrapper, prefix, **kwargs):
        ops = operators.all()
        rops = []
        for op in ops.values():
            rops.append({'operator': op.__name__,
                         'fullname': op.fullname,
                         'description': op.__doc__})
        return loader.render_to_string('instdata/operators.html', {'items': rops})




from decimal import Decimal


def topython(fname = 'qn'):
    f = open(fname+'.txt')
    r = f.read()
    f.close()
    rs = r.split('\n')
    pytxt = '%s = [' % fname
    vals  = []
    for r in rs:
        kv = r.split(' ')
        l  = []
        for k in kv:
            if k:
                d = Decimal(k)
                l.append(k)
        if len(l) != 2:
            break
        vals.append('      (%s)' % ', '.join(l))
    vals = ',\n'.join(vals)
    return '%s\n%s\n     ]' % (pytxt,vals)
    
    
    
if __name__ == '__main__':
    qn = topython('qn')
    qp = topython('qp')
    f = open('qpqn.py','w')
    qpqn = '%s\n\n%s' % (qp,qn)
    f.write(qpqn)
    f.close()


from dynts.conf import settings
import ply.lex as lex


class rules(object):
    
    # Regular expression rules for simple tokens
    t_PLUS    = r'\+'
    t_MINUS   = r'-'
    t_TIMES   = r'\*'
    t_DIVIDE  = r'/'
    t_LPAREN  = r'\('
    t_RPAREN  = r'\)'
    t_LSQUARE = r'\['
    t_RSQUARE = r'\]'
    t_EQUAL   = r'\='
    t_QUOTE   = r'\"'
    t_CONCAT  = r'\%s' % settings.concat_operator
    t_SPLIT   = r'\%s' % settings.separator_operator

    def __init__(self, oper = None):
        self.lexer = None
        self.oper  = oper or {}
        
    def reserved(self):
        return {}
    
    def __get_tokens(self):
        re = self.reserved()
        tokens = [
              'NUMBER',
              'PLUS',
              'MINUS',
              'TIMES',
              'DIVIDE',
              'LPAREN',
              'RPAREN',
              'LSQUARE',
              'RSQUARE',
              'EQUAL',
              'CONCAT',
              'SPLIT',
              'QUOTE',
              'ID',
              'FUNCTION'
              ] + re.values() 
        return tokens
    tokens = property(fget = __get_tokens)
    
    def __get_precedence(self):
        return (('left','SPLIT'),
                ('left','CONCAT'),
                ('left','EQUAL'),
                ('left','PLUS','MINUS'),
                ('left','TIMES','DIVIDE'),
                )
    precedence = property(fget = __get_precedence)

    # A regular expression rule with some action code
    def t_NUMBER(self, t):
        r'([0-9]+\.?[0-9]*|\.[0-9]+)([eE](\+|-)?[0-9]+)?'
        #r'\d+'
        try:
            v = float(t.value)
            iv = int(v)
            t.value = iv if iv == v else v
        except ValueError:
            print("Number %s is too large!" % t.value)
            t.value = 0
        return t

    def t_ID(self, t):
        r'[a-zA-Z_][a-zA-Z_0-9:@]*'
        res    = self.oper.get(t.value, None) # Check for reserved words (operations)
        if res == None:
            res = t.value.upper()
            if res == 'FALSE':
                r.type = 'BOOL'
                r.value = False
            elif res == 'TRUE':
                r.type = 'BOOL'
                r.value = True
            else:
                t.type = 'ID'
        else:
            t.value = res
            t.type  = 'FUNCTION'
        return t

    # Error handling rule
    def t_error(self, t):
        print "Illegal character '%s'" % t.value[0]
        t.lexer.skip(1)
    
    def build(self, **kwargs):
        self.lexer = lex.lex(object=self, **kwargs)

    def input(self, data):
        self.lexer.input(data)
    




from datetime import date, datetime, timedelta

from rpy2 import rinterface
from rpy2.robjects import r,numpy2ri

from dynts.exceptions import MissingPackage

def loadlib(lib):
    try:
        r('library(%s)' % lib)
    except rinterface.RRuntimeError:
        raise MissingPackage('R library %s not installed. From R shell type\n\ninstall.packages("%s")' % (lib,lib))

class rpyobject(object):
    _robj = None
    libraries = []
    
    def __new__(cls, *args, **kwargs):
        obj = super(rpyobject,cls).__new__(cls)
        obj.r = cls.load()
        return obj
    
    @classmethod
    def load(cls):
        if not cls._robj:
            cls._robj = r
            for lib in cls.libraries:
                loadlib(lib)
                    
        return cls._robj
    

# Convert to and From R date and python date
EPOCH = 1970
_EPOCH_ORD = date(EPOCH, 1, 1).toordinal()


def rdate0(dte):
    year, month, day = dte.timetuple()[:3]
    return date(year, month, 1).toordinal() - _EPOCH_ORD + day - 1

def rdate1(dte):
    year, month, day, hour, minute, second = dte.timetuple()[:6]
    days = date(year, month, 1).toordinal() - _EPOCH_ORD + day - 1
    return days + (hour + (minute + (second + 0.000001*dte.microsecond)/60.0)/60.0)/24.0

_converter = {datetime:rdate1,
              date:rdate0}

def py2rdate(dte):
    return _converter[dte.__class__](dte)


def r2pydate(tstamp):
    if not tstamp - round(tstamp):
        ordinal = _EPOCH_ORD + tstamp
        return date.fromordinal(ordinal)
    else:
        return datetime.fromordinal(tstamp)
    
    
def isoformat(dte):
    if isinstance(dte,datetime):
        raise NotImplementedError
    else:
        return dte.isoformat()
    


#
# NOT USED DIRECTLY - THIS MODULE IS IMPORTED BY dynts.tests
#
import unittest
import dynts


class SimpleFunctionTest(unittest.TestCase):
    function = None
    
    def testNoParameters(self):
        '''Test mean function with zero parameters'''
        expression = '%s(GOOG)' % self.function
        result = dynts.evaluate(expression)
        self.assertEqual(str(result),expression)
        self.assertEqual(len(result.data),1)
        data = result.ts()
        self.assertTrue(dynts.istimeseries(data))
        self.assertEqual(data.count(),1)
        
    def testWindowParameter(self):
        '''Test mean function with one parameter'''
        e = '%(f)s(GOOG,window=30),%(f)s(GOOG,window=60)' % {'f':self.function}
        result = dynts.evaluate(e)
        symbols = result.expression.symbols()
        self.assertEqual(len(symbols),1)
        self.assertEqual(len(result.data),1)
        data = result.ts()
        self.assertTrue(dynts.istimeseries(data))
        self.assertEqual(data.count(),2)
        
    def testTwoSeries(self):
        e = '%s(GOOG,YHOO)' % self.function
        result = dynts.evaluate(e)
        symbols = result.expression.symbols()
        self.assertEqual(len(symbols),2)
        self.assertEqual(len(result.data),2)
        data = result.ts()
        self.assertTrue(dynts.istimeseries(data))
        self.assertEqual(data.count(),2)
        


import unittest
from itertools import izip

import dynts
from dynts import dsl

class TestDsl(unittest.TestCase):
    names = ['eur','1eur','eu3r','eur4567']
    
    def testName(self):
        for name in self.names:
            res = dynts.parse(name)
            self.assertTrue(isinstance(res,dsl.Symbol))
            self.assertEqual(name.upper(),str(res))
        
    def testBinaryOperation(self):
        res = dynts.parse('2*GOOG')
        self.assertEqual(len(res.children),2)
        names = res.symbols()
        self.assertEqual(len(names),1)
        self.assertEqual(names[0],str(res.right))
        #ts = dynts.evaluate(res)
        #self.assertEqual(ts.count(),1)
    
    def testAdditionOperation(self):
        res = dynts.parse('YHOO+GOOG')
        self.assertEqual(len(res.children),2)
        names = res.symbols()
        self.assertEqual(len(names),2)
        self.assertEqual(names[0],str(res.left))
        self.assertEqual(names[1],str(res.right))
        
    def testTwoTimeSeries(self):
        '''Get a timeseries and a function and check for consistency'''
        res = dynts.parse('gs:yahoo,min(gs:yahoo,window=30)')
        self.assertEqual(res.symbols(),['GS:YAHOO'])
        self.assertEqual(len(res),2)
        self.assertEqual(str(res[0]),'GS:YAHOO')
        self.assertEqual(str(res[1]),'min(GS:YAHOO,window=30)')
        
    def testDataProvider(self):
        result = dynts.evaluate('2*GOOG,GOOG')
        self.assertEqual(len(result.data),1)
        self.assertEqual(result.expression,dynts.parse('2*GOOG,GOOG'))
        data = result.ts()
        self.assertTrue(dynts.istimeseries(data))
        self.assertEqual(data.count(),2)
        ts1 = data.serie(0)
        ts2 = data.serie(1)
        for v1,v2 in izip(ts1,ts2):
            self.assertAlmostEqual(v1,2.*v2)
        

        



from timeseries import *
from dsl import *
from dataprovider import *
from testdynts import *
from testflot import *
from testop import *
from testcrossection import *
from test_datastructures import *


import functions

class TestMeanFunction(functions.SimpleFunctionTest):
    function = 'ma'

class TestMaxFunction(functions.SimpleFunctionTest):
    function = 'max'
    
class TestMedFunction(functions.SimpleFunctionTest):
    function = 'med'

class TestMinFunction(functions.SimpleFunctionTest):
    function = 'min'



import os
import csv
from urllib import urlopen
try:
    import httplib2
except:
    httplib2 = None

from dynts.conf import settings

from base import DataProvider


short_month = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec')


class WebCsv(DataProvider):

    def __init__(self):
        if httplib2:
            self.h = httplib2.Http()
        else:
            self.h = None
        self.h = None
        
    def string_to_date(self, sdte):
        from ccy import dateFromString
        return dateFromString(sdte)
    
    def request(self, url):
        if self.h:
            resp, content = self.h.request(url)
            if resp.status == 200:
                return resp
        else:
            return urlopen(url, proxies = settings.proxies)
    
    def rowdata(self, ticker, startdate, enddate):
        url = self.hystory_url(str(ticker), startdate, enddate)
        res = self.request(url)
        return csv.DictReader(res)
        
    def hystory_url(self, ticker, startdate, enddate, field = None):
        raise NotImplementedError
    
    def allfields(self, ticker = None):
        return ['Close','Open','Low','High','Volume']
        
    def load(self, ticker, startdate, enddate, field, logger, backend, **kwargs):
        data = self.rowdata(ticker, startdate, enddate)
        if not data:
            return

        fields = {}        
        std  = self.string_to_date
        datestr = None
        dates = []
        for r in data:
            try:
                if not datestr:
                    val   = None
                    dt    = None
                    found = 0
                    for k,v in r.items():
                        if len(k) >= 4:
                            if k[len(k)-4:] == 'Date':
                                datestr = k
                                continue
                        fields[str(k).upper()] = []
                
                dt  = std(r[datestr])
                dates.append(dt)
                for k,v in r.items():
                    nts = fields.get(str(k).upper(),None)
                    if nts is not None:
                        try:
                            nts.append(float(v))
                        except:
                            continue
            except:
                continue
        
        field = field or 'CLOSE'
        return {'date': dates,
                'value': fields.get(str(field).upper(),None)}


class google(WebCsv):
    baseurl = 'http://finance.google.com/finance'
        
    def getdate(self, st, dte):
        m = short_month[dte.month-1]
        return '%s=%s+%s,+%s' % (st,m,dte.day,dte.year)
        
    def hystory_url(self, ticker, startdate, enddate, field = None):
        b = self.baseurl
        st = self.getdate('startdate', startdate)
        et = self.getdate('enddate', enddate)
        return '%s/historical?q=%s&%s&%s&output=csv' % (b,ticker,st,et)
    
    def weblink(self, ticker):
        return '%s?q=%s' % (self.baseurl,ticker)
    

class yahoo(WebCsv):
    baseurl = 'http://ichart.yahoo.com'
        
    def getdate(self, st, dte):
        return '%s=%s&%s=%s&%s=%s' % (st[0],dte.month-1,st[1],dte.day,st[2],dte.year)
        
    def hystory_url(self, ticker, startdate, enddate):
        b = self.baseurl
        st = self.getdate(('a','b','c'), startdate)
        et = self.getdate(('d','e','f'), enddate)
        return '%s/table.csv?s=%s&%s&%s&g=d&ignore=.csv' % (b,ticker,st,et)
    
    def weblink(self, ticker):
        return 'http://finance.yahoo.com/q?s=%s' % ticker





from distutils.core import setup
from distutils.command.install_data import install_data
from distutils.command.install import INSTALL_SCHEMES
import os
import sys

package_name = 'dynts'
root_dir     = os.path.dirname(__file__)
package_dir  = os.path.join(root_dir, package_name)


class osx_install_data(install_data):

    def finalize_options(self):
        self.set_undefined_options('install', ('install_lib', 'install_dir'))
        install_data.finalize_options(self)

if sys.platform == "darwin": 
    cmdclasses = {'install_data': osx_install_data} 
else: 
    cmdclasses = {'install_data': install_data}  

# Tell distutils to put the data_files in platform-specific installation
# locations. See here for an explanation:
# http://groups.google.com/group/comp.lang.python/browse_thread/thread/35ec7b2fed36eaec/2105ee4d9e8042cb
for scheme in INSTALL_SCHEMES.values():
    scheme['data'] = scheme['purelib']
 
def get_module():
    if root_dir not in sys.path:
        sys.path.insert(0,root_dir)
    return __import__(package_name)

mod = get_module()


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

def requirements():
    req = read('requirements.txt').replace('\r','').split('\n')
    result = []
    for r in req:
        r = r.replace(' ','')
        if r:
            result.append(r)
    return result 
 
def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)
 
# Compile the list of packages available, because distutils doesn't have
# an easy way to do this.
def get_rel_dir(d,base,res=''):
    if d == base:
        return res
    br,r = os.path.split(d)
    if res:
        r = os.path.join(r,res)
    return get_rel_dir(br,base,r)

packages, data_files = [], []
pieces = fullsplit(root_dir)
if pieces[-1] == '':
    len_root_dir = len(pieces) - 1
else:
    len_root_dir = len(pieces)

for dirpath, dirnames, filenames in os.walk(package_dir):
    # Ignore dirnames that start with '.'
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'): del dirnames[i]
    if '__init__.py' in filenames:
        packages.append('.'.join(fullsplit(dirpath)[len_root_dir:]))
    elif filenames:
        rel_dir = get_rel_dir(dirpath,root_dir)
        data_files.append([rel_dir, [os.path.join(dirpath, f) for f in filenames]])

if len(sys.argv) > 1 and sys.argv[1] == 'bdist_wininst':
    for file_info in data_files:
        file_info[0] = '\\PURELIB\\%s' % file_info[0]
        

setup(
        name         = package_name,
        version      = mod.__version__,
        author       = mod.__author__,
        author_email = mod.__contact__,
        url          = mod.__homepage__,
        license      = mod.__license__,
        description  = mod.__doc__,
        long_description = read('README.rst'),
        packages     = packages,
        cmdclass     = cmdclasses,
        data_files   = data_files,
        install_requires = requirements(),
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Environment :: Plugins',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: BSD License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Topic :: Scientific/Engineering',
            'Topic :: Scientific/Engineering :: Mathematics',
            'Topic :: Office/Business :: Financial'
        ],
    )
 



import unittest

import dynts


class TestInitFile(unittest.TestCase):

    def test_version(self):
        self.assertTrue(dynts.VERSION)
        self.assertTrue(dynts.__version__)
        self.assertEqual(dynts.__version__,dynts.get_version())
        self.assertTrue(len(dynts.VERSION) >= 2)

    def test_meta(self):
        for m in ("__author__", "__contact__", "__homepage__", "__doc__"):
            self.assertTrue(getattr(dynts, m, None))


from dynts.dsl.ast.astbase import *

class BinMathOp(BinOp):
    
    def __init__(self,left,right,op):
        super(BinMathOp,self).__init__(left,right,op)
    
    def dooper(self, le, ri):
        if isnumber(le) and isnumber(ri):
            return self.simpleoper(le,ri)
        else:
            return self.complexoper(le,ri)
    
    def _unwind(self, values, backend, full = False, **kwargs):
        le = self.left.unwind(values, backend, **kwargs)
        ri = self.right.unwind(values, backend, **kwargs)
        return self.dooper(le,ri)
    
    def lineardecomp(self):
        if isinstance(self.left,Number):
            if isinstance(self.right,Number):
                raise ValueError
            nv = self.left.value
            d1 = self.right.lineardecomp()
            dr = {}
            for k,v in d1.items():
                dr[k] = nv*v
            return dr
    
    def simpleoper(self,le,ri):
        '''Neeeds implementation'''
        pass
    
    def complexoper(self,le,ri):
        name = str(self)
        if isnumber(le):
            data = self.simpleoper(le, ri.values())
            return ri.clone(data = data, name = name)
        elif isnumber(ri):
            data = self.simpleoper(le.values(), ri)
            return le.clone(data = data, name = name)
        else:
            raise NotImplementedError
    

class PlusOp(BinMathOp):
    
    def __init__(self,left,right):
        BinMathOp.__init__(self,left,right,'+')
    
    def simpleoper(self, le, ri):
        return le+ri
    
    def lineardecomp(self):
        ls = self.left.lineardecomp()
        rs = self.right.lineardecomp()
        if ls and rs:
            lc = linearDecomp()
            lc.expand(ls).expand(rs)
            return lc
        else:
            return None
    
    
class MinusOp(BinMathOp):
    def __init__(self,left,right):
        BinMathOp.__init__(self,left,right,'-')

    def simpleoper(self, le, ri):
        return le-ri
    
    def lineardecomp(self):
        ls = self.left.lineardecomp()
        rs = self.right.lineardecomp()
        if ls and rs:
            lc = linearDecomp()
            lc.expand(ls).expand(rs.changesign())
            return lc
        else:
            return None
        
        
class MultiplyOp(BinMathOp):
    def __init__(self,left,right):
        BinMathOp.__init__(self,left,right,'*')
        
    def simpleoper(self, le, ri):
        return le*ri
    
    def lineardecomp(self):
        return linearDecomp().append(self)
    
    
class DivideOp(BinMathOp):
    
    def __init__(self,left,right):
        BinMathOp.__init__(self,left,right,'/')
        
    def simpleoper(self, le, ri):
        return le/ri
    
    def lineardecomp(self):
        return linearDecomp().append(self)




'''
This is just a landing zone for working on possible functionality that can be moved into the base timeseries class
'''

from zoo import TimeSeries
import operators

class TSWithOperators(TimeSeries):
    
    __add__ = operators.add
    __sub__ = operators.sub
    __mul__ = operators.mul
    __div__ = operators.div

#import datetime
#import operator

#class Analytics(object):
#    ops = {}
#
#    def __init__(self, ts):
#        self.ts = ts
#
#    def add_op(self, name, op):
#        '''
#        Add a new analytic operation
#        '''
#        ops = self.ops
#        ops[name] = op
#
#    def get_op(self, name):
#        ops = self.ops
#        op = ops.get(name, None)
#        if op is None:
#            op = getattr(operator, name, None)
#            if op is None:
#                msg = "No operation named %s. Choices are %s"
#                raise ValueError(msg %(name, ops))
#            else:
#                ops[name] = op
#        return op
#
#    def _ts_op(self, other_ts, op_name):
#        op = self.get_op(op_name)
#        ts = self.ts
#        all_dates = set(ts.keys()).union(set(other_ts.keys()))
#        new_ts = ts.copy()
#        new_ts.properties.merge(other_ts.properties.all())
#        for dt in all_dates:
#            val = ts.get(dt, None)
#            val2 = other_ts.get(dt, None)
#            if not(val is None or val2 is None):
#                new_val = op(val, val2)
#            else:
#                new_val = None
#            new_ts[dt] = new_val
#        return new_ts
#
#    def _scalar_op(self, scalar, op_name):
#        op = self.get_op(op_name)
#        ts = self.ts
#        all_dates = ts.keys()
#        new_ts = ts.copy()
#        for dt in all_dates:
#            val = ts.get(dt, None)
#            if val is None:
#                new_val = None
#            else:
#                new_val = op(val, scalar)
#            new_ts[dt] = new_val
#        return new_ts
#
#    def add(self, ts_or_scalar):
#        op = 'add'
#        result = self._ts_or_scalar_op(ts_or_scalar, op)
#        return result
#
#    def sub(self, ts_or_scalar):
#        op = 'sub'
#        result = self._ts_or_scalar_op(ts_or_scalar, op)
#        return result
#
#    def mult(self, ts_or_scalar):
#        op = 'mul'
#        result = self._ts_or_scalar_op(ts_or_scalar, op)
#        return result
#
#    def div(self, ts_or_scalar):
#        op = 'div'
#        result = self._ts_or_scalar_op(ts_or_scalar, op)
#        return result
#
#    def _ts_or_scalar_op(self, ts_or_scalar, op):
#        if isinstance(ts_or_scalar, TimeSeries):
#            new_ts = self._ts_op(ts_or_scalar, op)
#        else:
#            new_ts = self._scalar_op(ts_or_scalar, op)
#        return new_ts
#
#    def na_omit(self):
#        ts = self.ts
#        new_ts = ts.copy()
#        for dt, val in ts.items():
#            if not val is None:
#                new_ts[dt] = val
#        return new_ts
#
#class devTimeSeries(TimeSeries):
#    '''
#    At the moment this inherits from the zoo timeseries as it
#    has most of the functionality implemented
#
#    At a later stage some of the functionality developed here can be 
#    included in the base TimeSeries object
#    '''
#
#    def __init__(self,**kwargs):
#        self.analytics = Analytics(self)
#
#    def get(self, ky, dflt = None):
#        if ky in self.keys():
#            result = self[ky]
#        else:
#            result = dflt
#        return result
#
#    def keys(self):
#        kys = list(self._kys)
#        return kys
#
#    def values(self):
#        vals = list(self._vals)
#        return vals
#
#    def items(self):
#        kys = self.keys()
#        vals = self.values()
#        rt = zip(kys, vals)
#        return rt
#
#    def _getslice(self, slice):
#        kys = self._kys
#        vals = self._vals
#
#        start = slice.start or 0
#        stop = slice.stop or len(kys)
#        s_ky = self._convert_dt(start)
#        e_ky = self._convert_dt(stop)
#        s_indx = kys.index(s_ky)
#        e_indx =  kys.index(e_ky)
#
#        ky_seg = kys[s_indx:e_indx]
#        val_seg = vals[s_indx:e_indx]
#
#        new_ts = self.copy()
#        for ky, val in zip(ky_seg, val_seg):
#            new_ts[ky] = val
#        return new_ts
#
#    def __getitem__(self, key_or_slice):
#        kys = self._kys
#        vals = self._vals
#
#        is_slice = hasattr(key_or_slice, 'start')
#        if is_slice:
#            new_ts = self._getslice(key_or_slice)
#            return new_ts
#        else:
#            ckey = self._convert_dt(key_or_slice)
#            indx = kys.index(ckey)
#            val = vals[indx]
#            return val
#
#    def __setitem__(self, key, item):
#        ckey = self._convert_dt(key)
#        indx = self._greater_indx(ckey)
#        kys = self._kys
#        kys.insert(indx, ckey)
#
#        vals = self._vals
#        vals.insert(indx, item)
#
#    def __contains__(self, item):
#        key = self._convert_dt(item)
#        kys = self._kys
#        rt = key in kys
#        return rt
#
#    def __add__(self, item):
#        ts = self.analytics.add(item)
#        return ts
#
#    def __sub__(self, item):
#        ts = self.analytics.sub(item)
#        return ts
#
#    def __mul__(self, item):
#        ts = self.analytics.mult(item)
#        return ts
#
#    def __div__(self, item):
#        ts = self.analytics.div(item)
#        return ts






#
# NOT USED DIRECTLY - THIS CLASS IS IMPORTED IN dynts.tests.timeseries
#
import unittest
from datetime import date
from itertools import izip

from dynts import timeseries, evaluate, tsname
from dynts.utils import cross, asarray
from dynts.utils.populate import populate, datepopulate, randomts
from dynts.exceptions import *


class TestTS(unittest.TestCase):
    backend = None
    
    def getdata(self, size = 100, cols = 1, delta = 1, start = None):
        dates = datepopulate(size = size, delta = delta)
        data = populate(size = size, cols = cols)
        return dates,data
        
    def getts(self, returndata = False, delta = 1, cols = 1, size = 100):
        dates,data = self.getdata(size,cols,delta)
        ts   = timeseries(name = 'test', date = dates, data = data, backend = self.backend)
        self.assertEqual(ts.shape,(size,cols))
        self.assertEqual(len(ts),size)
        self.assertEqual(ts.count(),cols)
        if returndata:
            return ts,list(dates),list(data)
        else:
            return ts
            
    def _rollingTest(self, func):
        ts = self.getts(cols = 2)
        rollfun = 'roll%s' % func
        mts30 = getattr(ts,rollfun)(window = 30)
        mts60 = getattr(ts,rollfun)(window = 60)
        self.assertEqual(len(mts30),len(ts) - 29)
        self.assertEqual(len(mts60),len(ts) - 59)
        self.assertEqual(mts30.count(),2)
        self.assertEqual(mts60.count(),2)
        values = ts.values()
        v30 = mts30.values()
        date   = asarray(ts.dates())
        c = 0
        for dt,v in mts30.items():
            tst = ts.clone(date[c:c+30],values[c:c+30])
            self.assertEqual(dt,tst.end())
            tv = getattr(tst,func)()
            c += 1
            for a,b in izip(v,tv):
                self.assertAlmostEqual(a,b)
    
    def testInit(self):
        ts,dates,data = self.getts(True)
        self.assertEqual(ts.type,self.backend)
        self.assertEqual(len(ts),len(dates))
        for dt,dt1 in izip(dates,ts.dates()):
            self.assertEqual(dt,dt1)
        
    def test_isregular(self):
        ts = self.getts()
        self.assertTrue(ts.isregular())
            
    def test_frequency(self):
        ts = self.getts()
        f = ts.frequency()
        self.assertAlmostEqual(f,1)
    
    def testMax(self):
        ts  = self.getts()
        val = cross(ts.max())
        for v in ts.values():
            self.assertTrue(val >= v)
        ts = self.getts(cols = 2)
        val = cross(ts.max())
        for v in ts.values():
            self.assertTrue(val >= v)
    
    def testMin(self):
        ts  = self.getts()
        val = cross(ts.min())
        for v in ts.values():
            self.assertTrue(val <= v)
        ts = self.getts(cols = 2)
        val = cross(ts.min())
        for v in ts.values():
            self.assertTrue(val <= v)
        
    def testDates(self):
        ts = self.getts()
        start = ts.start()
        end = ts.end()
        self.assertTrue(end > start)
        sts = ts.window(start,end)
        self.assertEqual(len(ts),len(sts))
        dates = ts.keys()
        
    def testMerge(self):
        ts = self.getts()
        ts2 = self.getts(delta = 2, cols = 2)
        ts3 = ts.merge(ts2)
        self.assertTrue(len(ts3)>0)
        self.assertEqual(ts3.count(),3)
        
    def testMultivariate(self):
        ts = self.getts(cols = 2)
        va = ts[0]
        self.assertEqual(len(va),2)
        #names = ts.colnames()
        #self.assertEqual(len(names),1)
        
    def testRollingMin(self):
        self._rollingTest('min')
        
    def testRollingMax(self):
        self._rollingTest('max')
            
    def testBinaryTreeWrapper(self):
        '''Test included in documentation'''
        ts = randomts(cols = 2, start = date(2010,1,1), size = 50)
        dts = ts.asbtree()
        self.assertEqual(dts.shape,ts.shape)
        values = ts.values()
        self.assertEqual(dts[ts.start()].all(),values[0].all())
        self.assertEqual(dts[ts.end()].all(),values[49].all())
        
    def testHashWrapper(self):
        ts = randomts(cols = 2, start = date(2010,1,1), size = 50)
        dts = ts.ashash()
        self.assertEqual(dts.shape,ts.shape)
        self.assertFalse(dts.modified)
        dts[date(2009,2,1)] = [56.4,78.6]
        self.assertTrue(dts.modified)
        self.assertEqual(len(dts),len(ts)+1)
        dts = dts.getts()
        self.assertEqual(ts.type,dts.type)
        self.assertEqual(len(dts),len(ts)+1)
        self.assertEqual(dts.count(),ts.count())
        self.assertTrue(dts.isconsistent())
        
    def testCSVformatter(self):
        ts = randomts(name = tsname("serie1","serie2"),
                      cols = 2, start = date(2010,1,1), size = 50)
        self.assertEqual(ts.names(),['serie1','serie2'])
        csv = ts.dump('csv')
        self.assertTrue(csv)
        
    def testXLSformatter(self):
        ts = randomts(name = "serie1,serie2",
                      cols = 2, start = date(2010,1,1), size = 50)
        try:
            import xlwt
        except ImportError:
            try:
                csv = ts.dump('xls')
            except FormattingException:
                pass
        else:
            xls = ts.dump('xls')
            self.assertTrue(xls)
            
    def testPlot(self):
        ts = randomts(name = "serie1,serie2",
                      cols = 2, start = date(2010,1,1), size = 50)
        try:
            import matpotlib
        except ImportError:
            try:
                csv = ts.dump('plot')
            except FormattingException:
                pass
        else:
            plot = ts.dump('plot')
            self.assertTrue(plot)
        
    def testDslNames(self):
        res = evaluate('amzn:yahoo,min(amzn:yahoo)', backend = self.backend)
        self.assertEqual(str(res.expression),'AMZN:YAHOO,min(AMZN:YAHOO)')
        self.assertEqual(len(res.data),1)
        ts = res.ts()
        self.assertEqual(ts.count(),2)
        self.assertEqual(ts.names(),['AMZN:YAHOO','min(AMZN:YAHOO,window=20)'])
        


from UserDict import UserDict

from dynts.conf import settings
from dynts.exceptions import *


class FormatterDict(UserDict):
    
    def oftype(self, typ):
        '''Return a generator of formatters codes of type typ'''
        for key, val in self.items():
            if val.type == typ:
                yield key

Formatters = FormatterDict()


class DynData(object):
    '''Base class for data. It has two subclasses: the timeseries interface :class:`dynts.TimeSeries`
and :class:`dynts.xydata` for 2 dimensional data types.
An instance of this class contains a dataset of series. Each serie can be seen as an independent entity
which, nevertheless, can have close tights with other series in the dataset.

.. attribute:: name

    name of data object.
    
.. attribute:: info
    
    additional information regarding data object.
    
'''
    
    def __init__(self, name, info):
        self.name = str(name)
        self.info = info
        
    def __repr__(self):
        d = self.description()
        b = '%s:%s' % (self.__class__.__name__,self.__class__.type)
        if d:
            return '%s:%s' % (b,d)
        else:
            return b
    
    def __str__(self):
        return self.description()
    
    def description(self):
        return self.name
    
    def names(self):
        '''List of names for series in dataset. It will always return a list or names with
length given by :class:`dynts.DynData.count`.'''
        N = self.count()
        names = self.name.split(settings.splittingnames)[:N]
        n = 0
        while len(names) < N:
            n += 1
            names.append('unnamed%s' % n)
        return names
    
    def count(self):
        '''Number of series in dataset.'''
        raise NotImplementedError
    
    def series(self):
        '''Iterator over series in dataset.'''
        raise NotImplementedError
    
    def serie(self, index):
        '''Get serie data by column index.'''
        raise NotImplementedError
    
    def display(self):
        '''Nicely display self on the shell. Useful during prototyping and development.'''
        raise NotImplementedError
    
    def dump(self, format = None, **kwargs):
        '''Dump the timeseries using a specific :ref:`format <formatters>`.'''
        formatter = Formatters.get(format,None)
        if not format:
            return self.display()
        elif not formatter:
            raise FormattingException('Formatter %s not available' % format)
        else:
            return formatter(self,**kwargs)



class xyserie(object):
    
    def __init__(self, name = '', data = None, lines = True, points = False):
        self.points = points if lines else True
        self.lines = lines
        self.name = name
        self.data = data



class xydata(DynData):
    '''A :class:`dynts.DynData` class for 2-dimensional series of data.'''
    def __init__(self, name = '', data = None, info = None, **kwargs):
        super(xydata,self).__init__(name,info)
        self._series = []
        if data:
            self.add(xyserie(name=name,data=data,**kwargs))
        
    def add(self, data):
        from dynts import tsname
        if isinstance(data,xyserie):
            self._series.append(data)
        elif isinstance(data,self.__class__):
            for serie in data.series():
                self.name = tsname(self.name,serie.name)
                self._series.append(serie)
            
    def series(self):
        return self._series

    def count(self):
        return len(self._series)
    
    def serie(self, index):
        return self._series[index]
    
        
    